/*! Tripetto Collector 1.7.1-29 - Copyright (C) 2019 Tripetto B.V. - All Rights Reserved */

/// <reference path="tripetto.slots.d.ts" />
/// <reference path="tripetto.vfsm.d.ts" />
/// <reference path="tripetto.map.d.ts" />
/// <reference path="marblecore.markdown.d.ts" />
/// <reference path="marblecore.callback.d.ts" />
/// <reference path="marblecore.functional.d.ts" />
/// <reference path="marblecore.sha2.d.ts" />
/// <reference path="marblecore.l10n.d.ts" />
/// <reference path="marblecore.serializer.d.ts" />
/// <reference path="marblecore.hookup.d.ts" />

declare module "tripetto-collector" {
    /** Dependencies */
    import * as NSSlots from "@tripetto/slots";
    /** Contains the package name. */
    export const NAME: string;
    /** Contains the version number. */
    export const VERSION: string;
    /** Explicit exports */
    export {
        Context,
        Data,
        Errors,
        IDataValues,
        IMoment,
        Instance,
        ISnapshot,
        IVariable,
        Value,
        IOntologyResumeEvent as ICollectorResumeEvent
    } from "@tripetto/vfsm";
    export {
        ICondition,
        IConditionBlock,
        IDefinition,
        INode,
        INodeBlock
    } from "@tripetto/map";
    export { Kinds, Slot } from "@tripetto/slots";
    export { Condition } from "tripetto-collector/lib/conditions/condition";
    export {
        ConditionBlock
    } from "tripetto-collector/lib/conditions/blocks/block";
    export {
        ConditionBlocks
    } from "tripetto-collector/lib/conditions/blocks/blocks";
    export {
        IConditionBlockType,
        TConditionBlock
    } from "tripetto-collector/lib/conditions/blocks/type";
    export { Branch } from "tripetto-collector/lib/branches/branch";
    export { Cluster } from "tripetto-collector/lib/clusters/cluster";
    export { Collector } from "tripetto-collector/lib/collector";
    export { Node } from "tripetto-collector/lib/nodes/node";
    export { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    export {
        HeadlessBlock
    } from "tripetto-collector/lib/nodes/blocks/headless";
    export { NodeBlocks } from "tripetto-collector/lib/nodes/blocks/blocks";
    export {
        INodeBlockType,
        TNodeBlock
    } from "tripetto-collector/lib/nodes/blocks/type";
    export {
        IObservableNode
    } from "tripetto-collector/lib/storyline/observable";
    export {
        ICollectorChangeEvent,
        ICollectorRestartEvent
    } from "tripetto-collector/lib/events";
    export { TChanges } from "tripetto-collector/lib/changes";
    export { TModes } from "tripetto-collector/lib/modes";
    export { TStatus } from "tripetto-collector/lib/status";
    export {
        markdownify,
        markdownifyTo,
        markdownifyToString
    } from "tripetto-collector/lib/markdown";
    export { MarkdownFeatures, MarkdownTypes } from "@marblecore/markdown";
    export { Storyline } from "tripetto-collector/lib/storyline/storyline";
    export { IStoryline } from "tripetto-collector/lib/storyline/interface";
    export { Moment } from "tripetto-collector/lib/storyline/moment";
    export { IPage } from "tripetto-collector/lib/storyline/page";
    export { Import } from "tripetto-collector/lib/data/import";
    export { Export } from "tripetto-collector/lib/data/export";
    /** Decorators */
    export {
        tripetto,
        tripetto as block
    } from "tripetto-collector/lib/decorators/block";
    export {
        condition,
        IConditionHandler,
        ConditionResult
    } from "tripetto-collector/lib/decorators/condition";
    export {
        validator,
        IValidatorHandler,
        ValidatorResult
    } from "tripetto-collector/lib/decorators/validate";
    export {
        destroy,
        IDestroyHandler
    } from "tripetto-collector/lib/decorators/destroy";
    /** Slots */
    export import Slots = NSSlots;
    /** Implicit exports */
    export * from "@marblecore/callback";
    export * from "@marblecore/functional";
    export * from "@marblecore/sha2";
    export * from "@marblecore/l10n";
    /** Contracts */
    export * from "tripetto-collector/lib/contracts/styles";
    export * from "tripetto-collector/lib/contracts/l10n";
    export * from "tripetto-collector/lib/contracts/themes";
}

declare module "tripetto-collector/lib/conditions/condition" {
    /** Dependencies */
    import {
        Condition as ConditionFactory,
        Conditions,
        Context
    } from "@tripetto/vfsm";
    import { Callback } from "@marblecore/callback";
    import { Collector } from "tripetto-collector/lib/collector";
    import { ConditionResult } from "tripetto-collector/lib/decorators/condition";
    import { ICondition } from "@tripetto/map";
    export class Condition extends ConditionFactory<ICondition> {
        /**
         * Creates a new condition instance.
         * @param pConditions Reference to the conditions dispatcher.
         * @param pCondition Reference to the condition properties.
         */
        constructor(pConditions: Conditions, pCondition: ICondition);
        /** Retrieves the identifier of the condition. */
        readonly id: string;
        /** Reference to the collector. */
        readonly collector: Collector;
        /**
         * Invoked when the collector wants to verify the condition.
         * @param pContext Reference to the context.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous condition callback.
         * @return Returns the result of the condition or the callback reference if
         * the condition should be handled asynchronously. Returns `false` when no
         * block is available.
         * @event
         */
        onCondition(
            pContext: Context,
            pCallback: Callback<boolean>
        ): ConditionResult;
    }
}

declare module "tripetto-collector/lib/conditions/blocks/block" {
    /** Dependencies */
    import {
        IBranch,
        ICluster,
        ICondition,
        IConditionBlock,
        INode
    } from "@tripetto/map";
    import { Condition } from "tripetto-collector/lib/conditions/condition";
    import { Context, IVariable, ImmutableValue } from "@tripetto/vfsm";
    import { IConditionBlockType } from "tripetto-collector/lib/conditions/blocks/type";
    import { Kinds, Slot } from "@tripetto/slots";
    import { TSerializeTypes } from "@marblecore/serializer";
    export abstract class ConditionBlock<Properties extends {} = {}> {
        /** Contains the type identifier of the block. */
        static readonly identifier: string;
        /** Retrieves the block type. */
        readonly type: IConditionBlockType<ConditionBlock<Properties>>;
        /**
         * Creates a new condition block instance.
         * @param pCondition Reference to the condition.
         * @param pContext Reference to the context.
         */
        constructor(pCondition: Condition, pContext: Context);
        /** Retreieves the condition props. */
        readonly condition: Readonly<ICondition>;
        /** Retrieves the block props. */
        readonly props: Readonly<Properties & IConditionBlock>;
        /** Retrieves the node props. */
        readonly node: Readonly<INode> | undefined;
        /** Retrieves the cluster props. */
        readonly cluster: Readonly<ICluster> | undefined;
        /** Retrieves the branch props. */
        readonly branch: Readonly<IBranch> | undefined;
        /**
         * Retrieves a slot.
         * @param sId Specifies the slot identifier. If omitted the function
         * tries to find the slot that is attached to the condition block.
         * @return Returns the slot or `undefined` if the slot was not found.
         */
        protected slotOf<S extends Slot = Slot>(sId?: string): S | undefined;
        /**
         * Retrieves an immutable value. You can only retrieve values for the
         * slots of an attached node. If you want to retrieve values from other nodes
         * or for conditions without an attached node, use `immutableValueOf`.
         * @param pReference Optional parameter with a slot reference. If
         * omitted the function tries to find the slot that is attached to the
         * condition block.
         * @param sKind Specifies the slot kind.
         * @return Returns the mutable value or `undefined` if the value was not found.
         */
        protected valueOf<
            T extends TSerializeTypes,
            S extends Slot<T> = Slot<T>
        >(
            pReference?: Slot | string,
            sKind?: Kinds
        ): ImmutableValue<T, S> | undefined;
        /**
         * Retrieves an immutable value. You can retrieve immutable values for any node.
         * @param pReference Optional parameter with a slot instance or identifier. If
         * omitted the function tries to find the slot that is attached to the
         * condition block.
         * @return Returns the immutable value or `undefined` if the value was not found.
         */
        protected immutableValueOf<
            T extends TSerializeTypes,
            S extends Slot<T> = Slot<T>
        >(pReference?: Slot | string): ImmutableValue<T, S> | undefined;
        /**
         * Retrieves a variable.
         * @param pReference Specifies the slot reference or variable identifier.
         * @return Returns the variable or `undefined` if the variable was not found.
         */
        protected variableFor(pReference: Slot | string): IVariable | undefined;
    }
}

declare module "tripetto-collector/lib/conditions/blocks/blocks" {
    /** Dependencies */
    import { Condition } from "tripetto-collector/lib/conditions/condition";
    import { ConditionBlock } from "tripetto-collector/lib/conditions/blocks/block";
    import { Context } from "@tripetto/vfsm";
    import { TConditionBlock } from "tripetto-collector/lib/conditions/blocks/type";
    export class ConditionBlocks {
        /**
         * Registers a block.
         * @param pBlock Specifies the block.
         * @param sIdentifier Specifies the block type identifier.
         * @param pAliases Specifies type aliases for the block.
         */
        static register(
            pBlock: TConditionBlock,
            sIdentifier: string,
            pAliases: string[]
        ): void;
        /**
         * Provides a new instance of a condition block for the supplied condition.
         * @param pCondition Reference to the condition.
         * @param pContext Reference to the context.
         * @return Returns a condition block instance.
         */
        static create<T extends ConditionBlock>(
            pCondition: Condition,
            pContext: Context
        ): T | undefined;
    }
}

declare module "tripetto-collector/lib/conditions/blocks/type" {
    /** Dependencies */
    import { Condition } from "tripetto-collector/lib/conditions/condition";
    import { ConditionBlock } from "tripetto-collector/lib/conditions/blocks/block";
    import { Context } from "@tripetto/vfsm";
    export interface IConditionBlockType<T extends ConditionBlock> {
        /** Contains the block type identifier. */
        readonly identifier: string;
        /**
         * Creates a new block instance.
         * @param pCondition Reference to the condition.
         * @param pContext Reference to the context.
         */
        new (pCondition: Condition, pContext: Context): T;
    }
    export type TConditionBlock = IConditionBlockType<ConditionBlock>;
}

declare module "tripetto-collector/lib/branches/branch" {
    /** Dependencies */
    import { Branch as BranchFactory } from "@tripetto/vfsm";
    import { IBranch } from "@tripetto/map";
    import { Collector } from "tripetto-collector/lib/collector";
    import { Cluster } from "tripetto-collector/lib/clusters/cluster";
    export class Branch extends BranchFactory<IBranch> {
        /** Retrieves the identifier of the branch. */
        readonly id: string;
        /** Reference to the collector. */
        readonly collector: Collector;
        /** Reference to the cluster. */
        readonly cluster: Cluster;
        /**
         * Invoked when the branch condition is requested.
         * @return For now, the branch condition is always true.
         * @event
         */
        onCondition(): boolean;
    }
}

declare module "tripetto-collector/lib/clusters/cluster" {
    /** Dependencies */
    import { Await } from "@marblecore/callback";
    import { Collector } from "tripetto-collector/lib/collector";
    import { Context, State } from "@tripetto/vfsm";
    import { ICluster } from "@tripetto/map";
    export class Cluster extends State<ICluster> {
        /** Retrieves the identifier of the cluster. */
        readonly id: string;
        /** Reference to the collector. */
        readonly collector: Collector;
        /** Specifies the position of the cluster in the collector as a percentage. */
        readonly percentageInCollector: number;
        /**
         * Retrieves the validation result for the supplied context.
         * @param pContext Reference to the context.
         */
        getValidation(pContext: Context): "unknown" | "pass" | "fail";
        /**
         * Invoked when the enter condition for the cluster is requested.
         * @event
         */
        onEnterCondition(): boolean;
        /**
         * Invoked when the cluster is entered.
         * @param pContext Reference to the context.
         * @param pAwait Reference to the await pointer
         * @return Returns the await reference.
         * @event
         */
        onEnter(pContext: Context, pAwait: Await): Await;
        /**
         * Invoked when the cluster leave condition is requested.
         * @event
         */
        onLeaveCondition(): boolean;
        /**
         * Invoked when the cluster is left (only for paginated collectors).
         * @param pContext Reference to the context.
         * @event
         */
        onLeave(pContext: Context): void;
        /**
         * Invoked when the cluster is canceled (only for paginated collectors).
         * @param pContext Reference to the context.
         * @event
         */
        onCancel(pContext: Context): void;
        /**
         * Invoked when the cluster needs to be validated.
         * @event
         */
        onValidate(pContext: Context): boolean;
        /**
         * Invoked when the cluster is validated.
         * @param pContext Reference to the context.
         * @param sResult Specifies the validation state.
         * @param sType Specifies if the validation type.
         * @event
         */
        onValidated(
            pContext: Context,
            sResult: "fail" | "pass",
            sType: "initial" | "revalidate"
        ): void;
        /**
         * Invoked when the cluster starts or stops evaluating.
         * @param pContext Reference to the context.
         * @param bEvaluating Specifies if the cluster is evaluating.
         * @event
         */
        onEvaluate(pContext: Context, bEvaluating: boolean): void;
    }
}

declare module "tripetto-collector/lib/collector" {
    /** Dependencies */
    import { Branch } from "tripetto-collector/lib/branches/branch";
    import { IDefinition } from "@tripetto/map";
    import { Cluster } from "tripetto-collector/lib/clusters/cluster";
    import { Condition } from "tripetto-collector/lib/conditions/condition";
    import {
        Context,
        ISnapshot,
        IValueChangeEvent,
        Instance,
        Ontology
    } from "@tripetto/vfsm";
    import {
        ICollectorChangeEvent,
        TCollectorEvents
    } from "tripetto-collector/lib/events";
    import { Await } from "@marblecore/callback";
    import { IStoryline } from "tripetto-collector/lib/storyline/interface";
    import { Node } from "tripetto-collector/lib/nodes/node";
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    import { Storyline } from "tripetto-collector/lib/storyline/storyline";
    import { TChanges } from "tripetto-collector/lib/changes";
    import { TModes } from "tripetto-collector/lib/modes";
    import { TStatus } from "tripetto-collector/lib/status";
    export class Collector<
        T extends NodeBlock = NodeBlock,
        Data = undefined
    > extends Ontology<
        Cluster,
        Node<T>,
        Branch,
        Condition,
        IDefinition,
        TCollectorEvents
    > {
        /**
         * Invoked when the collector instance is changed.
         * @event
         */
        onChange?: (pEvent: ICollectorChangeEvent<T>) => void;
        /**
         * Invoked when the collected data of an instance is changed.
         * @event
         */
        onData?: (
            pInstance: Instance,
            pValueChangeEvent: IValueChangeEvent
        ) => void;
        /**
         * Invoked when the collector instance finishes.
         * @event
         */
        onFinish?: (pInstance: Instance) => Promise<void> | boolean;
        /**
         * Invoked when the collector is restarted.
         * @event
         */
        onRestart?: (pInstance: Instance) => void;
        /** Retrieves an initial storyline. */
        static getInitialStoryline<T extends NodeBlock = NodeBlock>(
            sMode: TModes
        ): IStoryline<T>;
        /**
         * Creates a new collector instance.
         * @param pDefinition Specifies the definition.
         * @param sMode Specifies the collector mode of operation. Can be one of the following values:
         * - `paginated`: Render each cluster on a separate page (this is the default behavior);
         * - `continuous`: Render all completed clusters and the current cluster on a page;
         * - `progressive`: Render all completed, current and future clusters on a page.
         * @param pStart Specifies if the collector should start immediately or
         * supplies the snapshot to restore.
         * @param bPreview Specifies if the collector should start in preview mode.
         */
        constructor(
            pDefinition: IDefinition | string,
            sMode?: TModes,
            pStart?: boolean | ISnapshot<Data>,
            bPreview?: boolean
        );
        /** Sets the definition. */
        definition: Readonly<IDefinition> | undefined;
        /** Retrieves a reference to the active instance. */
        readonly instance: Instance | undefined;
        /** Retrieves the storyline of the running instance. */
        readonly storyline: Storyline<T> | undefined;
        /** Retrieves the name of the definition. */
        readonly name: string;
        /** Sets the operation mode of the collector. */
        mode: TModes;
        /** Sets the preview mode. */
        preview: boolean;
        /** Retrieves the status of the collector. */
        readonly status: TStatus;
        /** Retrieves if the collector is empty. */
        readonly isEmpty: boolean;
        /**
         * Invoked when the collector is updated.
         * @param pInstance Reference to the instance.
         * @param sChange Specifies the type of change.
         * @param pStoryline Contains the storyline (if the instance is running).
         * @event
         */
        protected onInstanceChange(
            pInstance: Instance,
            sChange: TChanges,
            pStoryline: Storyline<T> | undefined
        ): void;
        /**
         * Invoked when an instance is started.
         * @param pInstance Reference to the instance.
         * @event
         */
        protected onInstanceStart(pInstance: Instance): void;
        /**
         * Invoked when the collected data of an instance is changed.
         * @param pInstance Reference to the instance.
         * @param pValueChangeEvent Reference to a value change event.
         * @event
         */
        protected onInstanceData(
            pInstance: Instance,
            pValueChangeEvent: IValueChangeEvent
        ): void;
        /**
         * Invoked when an instance is processing.
         * @param pInstance Reference to the instance.
         * @param bProcessing Specifies if the instance is processing.
         * @event
         */
        protected onInstanceProcessing(
            pInstance: Instance,
            bProcessing: boolean
        ): void;
        /**
         * Invoked when an instance is about to finish.
         * @param pInstance Reference to the instance.
         * @param fnFinish A callback function that can be used to control the finish process when you override this event and return `false`.
         * @return Return `true` if the instance finish is allowed or `false` if the instance finish should wait till the callback function is invoked.
         * @event
         */
        protected onInstanceFinishing(
            pInstance: Instance,
            fnFinish: (bFinish: boolean) => void
        ): boolean;
        /**
         * Invoked when an instance ends. This event is always invoked
         * when an instance is finished, stopped or paused.
         * @param pInstance Reference to the instance.
         * @param sType Specifies why the instance has ended. Can be one of the
         * following values:
         * - `finished`: The instance has finished;
         * - `stopped`: The instance was forced to stop;
         * - `paused`: The instance was paused.
         * @event
         */
        protected onInstanceEnd(
            pInstance: Instance,
            sType: "finished" | "stopped" | "paused"
        ): void;
        /**
         * Invoked when a cluster is entered.
         * @param pContext Reference to the context.
         * @param pCluster Reference to the cluster to render.
         * @param pObserver Reference to the await pointer that signals the
         * collector to validate the cluster and make a step forward. If the
         * collector type is set to `progressive` this is done automatically.
         * @event
         */
        onInstanceCluster(
            pContext: Context,
            pCluster: Cluster,
            pObserver: Await
        ): void;
        /**
         * Invoked when a cluster starts or stops evaluating.
         * @param pInstance Reference to the instance.
         * @param pCluster Reference to the cluster that is evaluating.
         * @param bEvaluating Specifies if the cluster is evaluating.
         * @event
         */
        onInstanceClusterEvaluating(
            pInstance: Instance,
            pCluster: Cluster,
            bEvaluating: boolean
        ): void;
        /**
         * Invoked when a cluster is left.
         * @param pInstance Reference to the instance.
         * @param sDirection Specifies the step direction.
         * @param pCluster Reference to the cluster that is left.
         * @event
         */
        onInstanceClusterLeave(
            pInstance: Instance,
            sDirection: "forward" | "backward",
            pCluster: Cluster
        ): void;
        /**
         * Invoked when a node is entered.
         * @param pContext Reference to the context.
         * @param pNode Reference to the node to render.
         * @param pObserver Reference to the await pointer that signals the
         * collector to validate the node. If the collector type is set to
         * `progressive` this is done automatically.
         * @event
         */
        onInstanceNode(
            pContext: Context,
            pNode: Node<T>,
            pObserver: Await
        ): void;
        /**
         * Invoked when the progress changes.
         * @param pInstance Reference to the instance.
         * @param nPercentage Contains the progress percentage.
         * @event
         */
        onInstanceProgress(pInstance: Instance, nPercentage: number): void;
        /**
         * Invoked when the instance makes a step.
         * @param pInstance Reference to the instance.
         * @param sDirection Specifies the step direction.
         * @event
         */
        onInstanceStep(
            pInstance: Instance,
            sDirection: "forward" | "backward"
        ): void;
        /**
         * Invoked when the cluster is validated.
         * @param pInstance Reference to the instance.
         * @param sResult Specifies the validation state.
         * @param sType Specifies if the validation type.
         * @event
         */
        onInstanceValidated(
            pInstance: Instance,
            sResult: "fail" | "pass",
            sType: "initial" | "revalidate"
        ): void;
        /**
         * Restores a collector snapshot.
         * @param pSnapshot Specifies the snapshot data.
         * @return Returns a reference to the instance.
         */
        restore(pSnapshot: ISnapshot<Data>): Instance | undefined;
        /**
         * Reloads the collector with the specified definition. During a reload
         * the collector tries to maintain any collected data.
         * @param pDefinition Specifies the definition.
         * @param bOnlyRestartWhenRunning Specifies to only restart the collector
         * if it was running when the reload was requested.
         * @return Returns the new instance.
         */
        reload(
            pDefinition: IDefinition,
            bOnlyRestartWhenRunning?: boolean
        ): Instance | undefined;
        /**
         * Restarts the collector.
         * @param bPreserveData Specifies if the data of an existing running
         * instance should be preserved.
         * @return Returns a reference to the instance.
         */
        restart(bPreserveData?: boolean): Instance;
        /**
         * Rerenders the collector for the specified instance.
         * @param pInstance Reference to the instance.
         */
        rerender(pInstance: Instance): void;
        /**
         * Step forward in the instance.
         * @return Returns `true` if the step succeeded.
         */
        stepForward(): boolean;
        /**
         * Step backward in the active instance.
         * @return Returns `true` if the step succeeded.
         */
        stepBackward(): boolean;
        /**
         * Step to the start of the active instance.
         * @return Returns `true` if the step succeeded.
         */
        stepToStart(): boolean;
        /**
         * Step to the head in the history of the active instance.
         * @return Returns `true` if the step succeeded.
         */
        stepToHead(): boolean;
        /**
         * Finish the active instance when it is finishable.
         * @return Returns a promise that resolves when the instance was finished.
         */
        finish(): Promise<void> | false;
    }
}

declare module "tripetto-collector/lib/nodes/node" {
    /** Dependencies */
    import { Await, Callback } from "@marblecore/callback";
    import { Cluster } from "tripetto-collector/lib/clusters/cluster";
    import { Collector } from "tripetto-collector/lib/collector";
    import { ConditionResult } from "tripetto-collector/lib/decorators/condition";
    import { Context, Transducer, Transducers } from "@tripetto/vfsm";
    import { INode } from "@tripetto/map";
    import { ValidatorResult } from "tripetto-collector/lib/decorators/validate";
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    import { HeadlessBlock } from "tripetto-collector/lib/nodes/blocks/headless";
    export class Node<
        T extends NodeBlock | HeadlessBlock = NodeBlock
    > extends Transducer<INode> {
        /**
         * Creates a new node instance.
         * @param pNodes Reference to the nodes dispatcher.
         * @param pNode Reference to the node properties.
         */
        constructor(pNodes: Transducers, pNode: INode);
        /** Retrieves the identifier of the node. */
        readonly id: string;
        /** Reference to the collector. */
        readonly collector: Collector;
        /** Reference to the cluster. */
        readonly cluster: Cluster;
        /**
         * Retrieves the block instance.
         * @param pContext Reference to the context.
         * @return Returns the block instance or `undefined` if the block was not found.
         */
        block(pContext: Context): T | undefined;
        /**
         * Invoked when the collector wants to verify the condition for the node.
         * This event is invoked just before the node is entered.
         * @param pContext Reference to the context.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous condition callback.
         * @return Returns the result of the condition or the callback reference if
         * the condition should be handled asynchronously.
         * @event
         */
        onEnterCondition(
            pContext: Context,
            pCallback: Callback<boolean>
        ): ConditionResult;
        /**
         * Invoked when the node is entered.
         * @param pContext Reference to the context.
         * @param pAwait Reference to the await pointer.
         * @event
         */
        onEnter(pContext: Context, pAwait: Await): Await;
        /**
         * Invoked when the node is left.
         * @param pContext Reference to the context.
         * @event
         */
        onLeave(pContext: Context): void;
        /**
         * Invoked when the transducer is canceled.
         * @param pContext Reference to the context.
         * @event
         */
        onCancel(pContext: Context): void;
        /**
         * Invoked when the node needs to be validated.
         * @param pContext Reference to the context.
         * @param sCurrent Specifies the current node validation state.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous validation callback.
         * @return Returns the result of the validation or the callback reference if
         * the validation should be handled asynchronously.
         * @event
         */
        onValidate(
            pContext: Context,
            sCurrent: "unknown" | "fail" | "pass",
            pCallback: Callback<boolean>
        ): ValidatorResult;
        /**
         * Invoked when the node is validated.
         * @param pContext Reference to the context.
         * @param sResult Specifies the validation result.
         * @param sType Specifies the validation type.
         * @event
         */
        onValidated(
            pContext: Context,
            sResult: "fail" | "pass",
            sType: "initial" | "revalidate"
        ): void;
    }
}

declare module "tripetto-collector/lib/nodes/blocks/block" {
    import { Context, IVariable, ImmutableValue, Value } from "@tripetto/vfsm";
    import { INode, INodeBlock } from "@tripetto/map";
    import { INodeBlockType } from "tripetto-collector/lib/nodes/blocks/type";
    import { Kinds, Slot } from "@tripetto/slots";
    import { Node } from "tripetto-collector/lib/nodes/node";
    import { TSerializeTypes } from "@marblecore/serializer";
    export abstract class NodeBlock<Properties extends {} = {}> {
        /** Contains the type name of the block. */
        static readonly identifier: string;
        /** Contains an optional ref. */
        static readonly ref?: {};
        /** Specifies if the block needs an automatic render upon value store. */
        static readonly autoRender: boolean;
        /** Specifies if the block needs automatic validation. */
        static readonly autoValidate: boolean;
        /** Retrieves the block type. */
        readonly type: INodeBlockType<NodeBlock<Properties>>;
        /** Reference to the context. */
        readonly context: Context;
        /**
         * Creates a new node block instance.
         * @param pNode Reference to the node.
         * @param pContext Reference to the context.
         */
        constructor(pNode: Node, pContext: Context);
        /** Retrieves the validation state of the block. */
        readonly validation: "unknown" | "pass" | "fail";
        /** Reference to the node props. */
        readonly node: Readonly<INode>;
        /** Contains the block props. */
        readonly props: Readonly<Properties & INodeBlock>;
        /** Specifies if an automatic render is needed upon value store. */
        readonly shouldAutoRender: boolean;
        /** Specifies if automatic validation is applied. */
        readonly shouldAutoValidate: boolean;
        /** Retrieves if the validation of the block failed. */
        readonly isFailed: boolean;
        /** Retrieves if the validation of the block passed. */
        readonly isPassed: boolean;
        /**
         * Retrieves a slot.
         * @param sReference Specifies the slot reference.
         * @param sKind Specifies the slot kind.
         * @return Returns the slot or `undefined` if the slot was not found.
         */
        protected slotOf<S extends Slot = Slot>(
            sReference: string,
            sKind?: Kinds
        ): S | undefined;
        /**
         * Retrieves a mutable value. You can only retrieve mutable values for the
         * slots managed by the block. If you want values from other nodes, use the
         * `immutableValueOf` method.
         * @param pReference Specifies the slot reference.
         * @param sKind Specifies the slot kind.
         * @return Returns the mutable value or `undefined` if the value was not found.
         */
        protected valueOf<
            T extends TSerializeTypes,
            S extends Slot<T> = Slot<T>
        >(pReference: Slot | string, sKind?: Kinds): Value<T, S> | undefined;
        /**
         * Retrieves an immutable value. You can retrieve immutable values for any node.
         * @param pReference Specifies the slot instance or identifier.
         * @return Returns the immutable value or `undefined` if the value was not found.
         */
        protected immutableValueOf<
            T extends TSerializeTypes,
            S extends Slot<T> = Slot<T>
        >(pReference: Slot | string): ImmutableValue<T, S> | undefined;
        /**
         * Retrieves a variable.
         * @param pReference Specifies the slot instance or variable identifier.
         * @return Returns the variable or `undefined` if the variable was not found.
         */
        protected variableFor(pReference: Slot | string): IVariable | undefined;
        /**
         * Retrieves a unique key for the block within the current context for the
         * specified label (the key is prefixed with an underscore). Use this function
         * if you need stable unique keys (for example for use as element identifiers).
         * @param sLabel Optional label for the key (if omitted the default key will
         * be returned).
         * @return Returns the key.
         */
        key(sLabel?: string): string;
        /** Invokes a block rerendering. */
        rerender(): void;
        /** Clears the data values of a block. */
        clear(): void;
    }
}

declare module "tripetto-collector/lib/nodes/blocks/headless" {
    /** Dependencies */
    import { Await } from "@marblecore/callback";
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    export abstract class HeadlessBlock<
        Properties extends {} = {}
    > extends NodeBlock<Properties> {
        /**
         * Implements the operation for the block.
         * @param pDone Reference to an await pointer that can be used for asynchronous operations.
         */
        abstract do(pDone: Await): void;
    }
}

declare module "tripetto-collector/lib/nodes/blocks/blocks" {
    /** Dependencies */
    import { Context } from "@tripetto/vfsm";
    import { Node } from "tripetto-collector/lib/nodes/node";
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    import { HeadlessBlock } from "tripetto-collector/lib/nodes/blocks/headless";
    import { TNodeBlock } from "tripetto-collector/lib/nodes/blocks/type";
    export class NodeBlocks {
        /**
         * Registers the supplied block.
         * @param pBlock Specifies the block.
         * @param sIdentifier Specifies the block type identifier.
         * @param pAliases Specifies type aliases for the block.
         * @param pRef Specifies an optional ref to something.
         * @param bAutoRender Specifies if the block needs to be rerendered upon
         * value store (disabled by default).
         * @param bAutoValidate Specifies if the block checks if all required
         * slots contain a value (enabled by default).
         */
        static register(
            pBlock: TNodeBlock,
            sIdentifier: string,
            pAliases: string[],
            pRef?: {},
            bAutoRender?: boolean,
            bAutoValidate?: boolean
        ): void;
        /**
         * Creates a new instance of a node block for the supplied node.
         * @param pNode Reference to the node.
         * @param pContext Reference to the context.
         * @return Returns a node block instance.
         */
        static create<T extends NodeBlock | HeadlessBlock>(
            pNode: Node<T>,
            pContext: Context
        ): T | undefined;
    }
}

declare module "tripetto-collector/lib/nodes/blocks/type" {
    /** Dependencies */
    import { Context } from "@tripetto/vfsm";
    import { Node } from "tripetto-collector/lib/nodes/node";
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    export interface INodeBlockType<T extends NodeBlock> {
        /** Contains the block type identifier. */
        readonly identifier: string;
        /** Optional reference to something. */
        readonly ref?: {};
        /** Specifies if the block needs an automatic render upon value store. */
        readonly autoRender?: boolean;
        /** Specifies if the block needs automatic validation. */
        readonly autoValidate?: boolean;
        /**
         * Creates a new block instance.
         * @param pNode Reference to the node.
         * @param pContext Reference to the context.
         */
        new (pNode: Node, pContext: Context): T;
    }
    export type TNodeBlock = INodeBlockType<NodeBlock>;
}

declare module "tripetto-collector/lib/storyline/observable" {
    /** Dependencies */
    import { Await } from "@marblecore/callback";
    import { Context } from "@tripetto/vfsm";
    import { INode } from "@tripetto/map";
    import { Node } from "tripetto-collector/lib/nodes/node";
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    export interface IObservableNode<T extends NodeBlock> {
        /** Contains a unique identifier for the node. */
        readonly id: string;
        /** Contains a unique key for the node within the current context (the key is prefixed with an underscore). */
        readonly key: string;
        /** Reference to the node. */
        readonly node: Node;
        /** Reference to the node properties. */
        readonly props: INode;
        /** Reference to the node block. */
        readonly block: T | undefined;
        /** Reference to the context. */
        readonly context: Context;
        /**
         * Contains the enumerator for the node. Enumerators are only granted to nodes
         * that have a block that collects data.
         */
        readonly enumerator?: number;
        /** Contains the validation state of the node. */
        readonly validation: "unknown" | "pass" | "fail";
        /** Retrieves if the validation of the node failed. */
        readonly isFailed: boolean;
        /** Retrieves if the validation of the node passed. */
        readonly isPassed: boolean;
        /** Reference to the node observer. */
        readonly observer?: Await;
    }
}

declare module "tripetto-collector/lib/events" {
    /** Dependencies */
    import { Collector } from "tripetto-collector/lib/collector";
    import { IHookPayload, THooks } from "@marblecore/hookup";
    import { Instance } from "@tripetto/vfsm";
    import { IStoryline } from "tripetto-collector/lib/storyline/interface";
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    import { TChanges } from "tripetto-collector/lib/changes";
    export type TCollectorEvents = "OnChange" | "OnRestart" | "OnReload";
    interface ICollectorEvent<H extends THooks, T extends NodeBlock, Data>
        extends IHookPayload<H> {
        /** Reference to the collector. */
        readonly collector: Collector<T, Data>;
    }
    export interface ICollectorChangeEvent<
        T extends NodeBlock = NodeBlock,
        Data = undefined
    > extends ICollectorEvent<"OnChange", T, Data> {
        /** Specifies the type of change. */
        readonly change: TChanges;
        /** Reference to the instance. */
        readonly instance: Instance | undefined;
        /** Reference to the storyline. */
        readonly storyline: IStoryline<T>;
    }
    export interface ICollectorRestartEvent<
        T extends NodeBlock = NodeBlock,
        Data = undefined
    > extends ICollectorEvent<"OnRestart", T, Data> {
        /** Reference to the instance. */
        readonly instance: Instance;
    }
    export {};
}

declare module "tripetto-collector/lib/changes" {
    export type TChanges =
        | "started"
        | "render"
        | "evaluating"
        | "validated"
        | "forward"
        | "backward"
        | "progress"
        | "finishing"
        | "finished"
        | "stopped"
        | "paused"
        | "mode"
        | "preview";
}

declare module "tripetto-collector/lib/modes" {
    export type TModes = "paginated" | "continuous" | "progressive";
}

declare module "tripetto-collector/lib/status" {
    export type TStatus =
        | "idle"
        | "empty"
        | "preview"
        | "running"
        | "paused"
        | "stopped"
        | "finishing"
        | "finished";
}

declare module "tripetto-collector/lib/markdown" {
    /** Dependencies */
    import {
        MarkdownFeatures,
        MarkdownParser,
        MarkdownTypes
    } from "@marblecore/markdown";
    import { Context, IVariable } from "@tripetto/vfsm";
    /**
     * Runs the string through the markdown parser.
     * @param sMarkdown Specifies the markdown string to parse.
     * @param pContext Specifies the context.
     * @param pOptions Specifies the markdown options.
     * @return Returns a markdown parser instance.
     */
    export function markdownify(
        sMarkdown: string,
        pContext: Context,
        pOptions?: {
            /** Specifies the supported markdown features. */
            features?: MarkdownFeatures;
            /** Specifies the placeholder for empty variables. */
            placeholder?: string;
            /** Specifies if line breaks are supported. */
            lineBreaks?: boolean;
        }
    ): MarkdownParser<IVariable>;
    /**
     * Parses a markdown string to a plain text string.
     * @param sMarkdown Specifies the markdown string to parse.
     * @param pContext Specifies the context.
     * @param sPlaceholder Specifies a string for empty variables.
     * @param bLineBreaks Specifies if line breaks are supported.
     * @return Returns a plain text string.
     */
    export function markdownifyToString(
        sMarkdown: string,
        pContext: Context,
        sPlaceholder: string,
        bLineBreaks?: boolean
    ): string;
    /**
     * Reduces or maps a markdown string to a certain structure.
     * @param sMarkdown Specifies the markdown string to parse.
     * @param pContext Specifies the context.
     * @param pOptions Specifies the markdown options.
     * @return Returns the parsed structure.
     */
    export function markdownifyTo<T>(
        sMarkdown: string,
        pContext: Context,
        pOptions: (
            | {
                  reduce: (
                      sType: MarkdownTypes | undefined,
                      pContent: string | T[],
                      pValue: string | IVariable | undefined
                  ) => T;
              }
            | {
                  map: (
                      sType: MarkdownTypes | undefined,
                      sContent: string,
                      pValue: string | IVariable | undefined,
                      pParent: T | undefined,
                      sKind: "root" | "text" | "node"
                  ) => T;
              }) & {
            /** Specifies the supported markdown features. */
            features?: MarkdownFeatures;
            /** Specifies the placeholder for empty variables. */
            placeholder?: string;
            /** Specifies if line breaks are supported. */
            lineBreaks?: boolean;
        }
    ): T;
}

declare module "tripetto-collector/lib/storyline/storyline" {
    /** Dependencies */
    import { Cluster } from "tripetto-collector/lib/clusters/cluster";
    import { Instance } from "@tripetto/vfsm";
    import { IObservableNode } from "tripetto-collector/lib/storyline/observable";
    import { IPage } from "tripetto-collector/lib/storyline/page";
    import { IStoryline } from "tripetto-collector/lib/storyline/interface";
    import { IStorylineOrchestrator } from "tripetto-collector/lib/storyline/orchestrator";
    import { Moment } from "tripetto-collector/lib/storyline/moment";
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    import { TModes } from "tripetto-collector/lib/modes";
    export class Storyline<T extends NodeBlock = NodeBlock> {
        /**
         * Creates a new storyline.
         * @param pOrchestrator Reference to the storyline orchestrator.
         */
        constructor(pOrchestrator: IStorylineOrchestrator);
        /** Retrieves the storyline checksum. */
        readonly checksum: string | undefined;
        /** Retrieves an immutable representation of the storyline. */
        readonly immutable: IStoryline<T>;
        /** Retrieves the instance of the storyline. */
        readonly instance: Instance;
        /** Retrieves the collection of moments. */
        readonly moments: Moment<T>[];
        /** Retrieves the collection of pages (only available in paginated mode). */
        readonly pages: IPage[];
        /** Retrieves the present moment. */
        readonly presentMoment: Moment<T> | undefined;
        /** Contains the cluster of the present moment. */
        readonly presentCluster: Cluster | undefined;
        /** Contains the observable nodes of the present moment. */
        readonly presentNodes: IObservableNode<T>[];
        /** Contains the observable nodes of the active moments. */
        readonly activeNodes: IObservableNode<T>[];
        /** Retrieves if the storyline is evaluating. */
        readonly isEvaluating: boolean;
        /** Retrieves if the storyline has changed since the last call of this prop. */
        readonly isChanged: boolean;
        /** Retrieves if the storyline is empty (has no observable nodes). */
        readonly isEmpty: boolean;
        /** Retrieves if the present moment is at the begin of the storyline. */
        readonly isAtStart: boolean;
        /** Retrieves if the present moment is at the head of the storyline. */
        readonly isAtHead: boolean;
        /** Retrieves if the present moment is at the finish of the storyline. */
        readonly isAtFinish: boolean;
        /** Retrieves if the storyline is finishable using the `finish()` method. */
        readonly isFinishable: boolean;
        /** Retrieves if the storyline is finishing. */
        readonly isFinishing: boolean;
        /** Retrieves if the storyline is finsihed. */
        readonly isFinished: boolean;
        /** Retrieves if the storyline has any data collected. */
        readonly hasDataCollected: boolean;
        /** Retrieves if the validation of the storyline failed. */
        readonly isFailed: boolean;
        /** Retrieves if the validation of the storyline passed. */
        readonly isPassed: boolean;
        /** Retieves the mode of operation. */
        readonly mode: TModes;
        /** Retrieves the most recent step direction of the storyline. */
        readonly direction: "forward" | "backward";
        /** Total number of observable nodes in the storyline. */
        readonly count: number;
        /** Total number of observable nodes that have an enumerator value. */
        readonly enumerators: number;
        /** Retrieves the progress percentage of the storyline. */
        readonly percentage: number;
        /** Invalidates the current storyline. */
        invalidate(): void;
        /** Resets the storyline and clears the moments cache. */
        reset(): void;
        /**
         * Maps moments to a certain type.
         * @param fnMoment Specifies the function to invoke for each moment.
         */
        map<R>(fnMoment: (pMoment: Moment<T>, nIndex: number) => R): R[];
        /**
         * Step forward in the storyline.
         * @return Returns `true` if the step succeeded.
         */
        stepForward(): boolean;
        /**
         * Step backward in the storyline.
         * @return Returns `true` if the step succeeded.
         */
        stepBackward(): boolean;
        /**
         * Step to the start of the storyline.
         * @return Returns `true` if the step succeeded.
         */
        stepToStart(): boolean;
        /**
         * Step to the head of the storyline.
         * @return Returns `true` if the step succeeded.
         */
        stepToHead(): boolean;
        /**
         * Steps to a page.
         * @param nPage Specifies the page to step to.
         * @return Returns `true` if the step succeeded.
         */
        stepToPage(nPage: number): boolean;
        /**
         * Finish the storyline when it is at the finish.
         * @return Returns a promise that resolves when the instance was finished.
         */
        finish(): Promise<void>;
        /**
         * Find the node that belongs to the supplied key.
         * @param sKey Specifies the key to use.
         * @return Returns the node of `undefined` if the node is not found.
         */
        getNodeByKey(sKey: string): IObservableNode<T> | undefined;
        /**
         * Finds the first key for the supplied node identifier.
         * @param sId Specifies the node id.
         * @return Returns the key of the node or `undefined` if the node is not found.
         */
        getKeyByNodeId(sId: string): string | undefined;
    }
}

declare module "tripetto-collector/lib/storyline/interface" {
    /** Dependencies */
    import { Cluster } from "tripetto-collector/lib/clusters/cluster";
    import { IObservableNode } from "tripetto-collector/lib/storyline/observable";
    import { IPage } from "tripetto-collector/lib/storyline/page";
    import { Moment } from "tripetto-collector/lib/storyline/moment";
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    import { Storyline } from "tripetto-collector/lib/storyline/storyline";
    import { TModes } from "tripetto-collector/lib/modes";
    export interface IStory<T extends NodeBlock> {
        /** Storyline instance. */
        readonly storyline: Storyline<T>;
        /** Storyline direction. */
        direction?: "forward" | "backward";
        /** Storyline checksum. */
        checksum?: string;
    }
    export interface IStoryline<T extends NodeBlock = NodeBlock> {
        /** Retrieves the collection of moments. */
        readonly moments: Moment<T>[];
        /** Retrieves the collection of pages (only available in paginated mode). */
        readonly pages: IPage[];
        /** Retrieves the present moment. */
        readonly presentMoment: Moment<T> | undefined;
        /** Contains the cluster of the present moment. */
        readonly presentCluster: Cluster | undefined;
        /** Contains the observable nodes of the present moment. */
        readonly presentNodes: IObservableNode<T>[];
        /** Contains the observable nodes of the active moments. */
        readonly activeNodes: IObservableNode<T>[];
        /** Retrieves if the storyline is evaluating. */
        readonly isEvaluating: boolean;
        /** Retrieves if the storyline is empty (has no observable nodes). */
        readonly isEmpty: boolean;
        /** Retrieves if the present moment is at the begin of the storyline. */
        readonly isAtStart: boolean;
        /** Retrieves if the present moment is at the head of the storyline. */
        readonly isAtHead: boolean;
        /** Retrieves if the present moment is at the finish of the storyline. */
        readonly isAtFinish: boolean;
        /** Retrieves if the storyline is finishable using the `finish()` method. */
        readonly isFinishable: boolean;
        /** Retrieves if the storyline is finishing. */
        readonly isFinishing: boolean;
        /** Retrieves if the storyline is finsihed. */
        readonly isFinished: boolean;
        /** Retrieves if the storyline has any data collected. */
        readonly hasDataCollected: boolean;
        /** Retrieves if the validation of the storyline failed. */
        readonly isFailed: boolean;
        /** Retrieves if the validation of the storyline passed. */
        readonly isPassed: boolean;
        /** Retieves the mode of operation. */
        readonly mode: TModes;
        /** Retrieves the most recent step direction of the storyline. */
        readonly direction: "forward" | "backward";
        /** Total number of observable nodes in the storyline. */
        readonly count: number;
        /** Total number of observable nodes that have an enumerator value. */
        readonly enumerators: number;
        /** Retrieves the progress percentage of the storyline. */
        readonly percentage: number;
        /** Maps moments to a certain type. */
        readonly map: <R>(
            fnMoment: (pMoment: Moment<T>, nIndex: number) => R
        ) => R[];
        /** Step forward in the storyline. */
        readonly stepForward: () => boolean;
        /** Step backward in the storyline. */
        readonly stepBackward: () => boolean;
        /** Step to the start of the storyline. */
        readonly stepToStart: () => boolean;
        /** Step to the head of the storyline. */
        readonly stepToHead: () => boolean;
        /** Steps to a page. */
        readonly stepToPage: (nPage: number) => boolean;
        /** Finish the storyline when it is at the finish. */
        readonly finish: () => Promise<void> | false;
    }
}

declare module "tripetto-collector/lib/storyline/moment" {
    import { Cluster } from "tripetto-collector/lib/clusters/cluster";
    import { IMoment } from "@tripetto/vfsm";
    import { IObservableNode } from "tripetto-collector/lib/storyline/observable";
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    export class Moment<T extends NodeBlock = NodeBlock> {
        /**
         * Creates a new moment.
         * @param pMoment Reference to the moment.
         * @param nEnumerator Specifies the enumerator value.
         */
        constructor(pMoment: IMoment, nEnumerator: number);
        /** Reference to the cluster. */
        readonly cluster: Cluster;
        /** Collection of nodes. */
        readonly nodes: IObservableNode<T>[];
        /** Retrieves the tense of the moment. */
        readonly tense: "past" | "present" | "future";
        /** Specifies if the moment is in the past. */
        readonly isPast: boolean;
        /** Specifies if the moment is the present moment. */
        readonly isPresent: boolean;
        /** Specifies if the moment is in the future. */
        readonly isFuture: boolean;
        /** Retrieves if the moment is evaluating. */
        readonly isEvaluating: boolean;
        /** Retrieves if the validation of the cluster failed. */
        readonly isFailed: boolean;
        /** Retrieves if the validation of the cluster passed. */
        readonly isPassed: boolean;
        /** Total number of nodes in the moment. */
        readonly count: number;
        /** Total number of nodes that have an enumerator value. */
        readonly enumerators: number;
        /** Restores the moment. */
        restore(): void;
    }
}

declare module "tripetto-collector/lib/storyline/page" {
    export interface IPage {
        /** Contains the page number. */
        readonly number: number;
        /** Contains if the page is active. */
        readonly active: boolean;
        /** Activates the page. */
        readonly activate: () => void;
    }
}

declare module "tripetto-collector/lib/data/import" {
    /** Dependencies */
    import { Context, IDataImport, Instance } from "@tripetto/vfsm";
    import { TSerializeTypes } from "@marblecore/serializer";
    export namespace Import {
        interface IDataValues {
            /** Data slot values. */
            readonly values: IDataImport;
        }
        /** Describes the values object. */
        interface IValues {
            readonly [slot: string]: IDataValues | undefined;
        }
        /** Describes an importable field value. */
        interface IFieldValue {
            /** Contains the data value. */
            readonly value: TSerializeTypes;
            /** Contains the data reference. */
            readonly reference?: string;
            /** Contains the UTC set time of the data. */
            readonly time?: number;
        }
        /** Describes an importable field by its key. */
        interface IFieldByKey extends IFieldValue {
            /** Key of the data field. */
            readonly key: string;
        }
        /** Describes an importable field by name. */
        interface IFieldByName extends IFieldValue {
            /** Contains the name. */
            readonly name: string;
        }
        /**
         * Import data values to the instance.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pValues Specifies the values to import.
         * @param pSelection Specifies the identifiers of the slots that should be imported.
         * @return Returns `true` if the import succeeded.
         */
        function values(
            pInstanceOrContext: Instance | Context,
            pValues: IValues,
            pSelection?: string[]
        ): boolean;
        /**
         * Import data fields to the instance.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pFields Contains the fields to import. You can reference a field
         * by its key or field name.
         * @param sSeparator Specifies the string that is used to separate multiple
         * field names (defaults to ` / `).
         * @param pSelection Specifies the identifiers of the slots that should be imported.
         * @return Returns `true` if the import succeeded.
         */
        function fields(
            pInstanceOrContext: Instance | Context,
            pFields: (IFieldByKey | IFieldByName)[],
            pSelection?: string[],
            sSeparator?: string
        ): boolean;
        /**
         * Imports CSV data. The number of comma-separated fields and their order
         * should exactly match the number of fields and their order in the instance.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param sRecord Specifies the CSV record to import.
         * @param pSelection Specifies the identifiers of the slots that should be imported.
         * @return Returns `true` if the import succeeded.
         */
        function CSV(
            pInstanceOrContext: Instance | Context,
            sRecord: string,
            pSelection?: string[]
        ): boolean;
    }
}

declare module "tripetto-collector/lib/data/export" {
    /** Dependencies */
    import { Context, IDataValues, Instance } from "@tripetto/vfsm";
    import { TSerializeTypes } from "@marblecore/serializer";
    export namespace Export {
        /** Describes the values object. */
        interface IValues {
            readonly [slot: string]:
                | {
                      /** Unique fingerprint of the definition. */
                      readonly fingerprint: string;
                      /** Node */
                      readonly node: {
                          /** Node identifier. */
                          readonly id: string;
                          /** Friendly name of node. */
                          readonly name: string;
                      };
                      /** Block */
                      readonly block: {
                          /** Contains the block type identifier. */
                          readonly type: string;
                          /** Contains the block version. */
                          readonly version: string;
                      };
                      /** Slot. */
                      readonly slot: {
                          /** Slot identifier. */
                          readonly id: string;
                          /** Reference of slot. */
                          readonly reference: string;
                          /** Alias or name of slot. */
                          readonly name: string;
                          /** Slot label. */
                          readonly label: string;
                      };
                      /** Data slot values. */
                      readonly values: IDataValues;
                  }
                | undefined;
        }
        /** Describes an exportable field. */
        interface IField {
            /** A unique key for the data field. */
            readonly key: string;
            /** Contains the name. */
            readonly name: string;
            /** Contains the block type identifier. */
            readonly type: string;
            /** Contains the block version. */
            readonly version: string;
            /** Contains the mode id. */
            readonly node: string;
            /** Contains the slot reference. */
            readonly slot: string;
            /** Contains the data as a string. */
            readonly string: string;
            /** Contains the data value. */
            readonly value: TSerializeTypes;
            /** Contains if the data was modified. */
            readonly modified: boolean;
            /** Contains the data reference. */
            readonly reference?: string;
            /** Contains the UTC set time of the data. */
            readonly time?: number;
        }
        interface IFields {
            /** Unique fingerprint of the definition. */
            readonly fingerprint: string;
            /** Contains the fields. */
            readonly fields: IField[];
        }
        /** Describes a CSV export. */
        interface ICSV {
            /** Unique fingerprint of the definition. */
            readonly fingerprint: string;
            /** Contains the CSV fields. */
            readonly fields: string;
            /** Contains the CSV record. */
            readonly record: string;
        }
        /**
         * Retrieves a collection of all valid exportable values. The values are stored
         * per slot. The key of each value object is the identifier of the slot. The keys
         * for the contextual values are composed using the condition hashes of the
         * context. If the context is global the key of that value is `*`. If the
         * context array contains a single condition, the key is the computed
         * hash of the stringified condition data (where the `id` property is set to
         * an empty `string`). If the context array contains 2 or more conditions, the
         * computed hashes of these conditions are used to calculate a new hash. The
         * computed condition hashes are concatenated in chronological order and then a
         * `SHA2_256` hash is generated for this concatenated string. This hash is
         * used as key.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pSelection Specifies the identifiers of the slots that should be exported.
         * @return Returns the values.
         */
        function values(
            pInstanceOrContext: Instance | Context,
            pSelection?: string[]
        ): IValues;
        /**
         * Retrieves an array with all data fields This includes all slots, even
         * the ones that don't contain data. So this array always has the
         * same number of items regardless of the actual collected slots. If you
         * just want the slots with data, use `fieldsFilled`.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pSelection Specifies the identifiers of the slots that should be exported.
         * @param sSeparator Specifies the string that is used to separate multiple
         * field names (defaults to ` / `).
         * @returns Returns an array with fields.
         */
        function fields(
            pInstanceOrContext: Instance | Context,
            pSelection?: string[],
            sSeparator?: string
        ): IFields;
        /**
         * Retrieves an array with only the filled fields.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pSelection Specifies the identifiers of the slots that should be exported.
         * @param sSeparator Specifies the string that is used to separate multiple
         * field names (defaults to ` / `).
         * @returns Returns an array with fields.
         */
        function fieldsFilled(
            pInstanceOrContext: Instance | Context,
            pSelection?: string[],
            sSeparator?: string
        ): IFields;
        /**
         * Retrieves a diff array with only the fields that have changed.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pSelection Specifies the identifiers of the slots that should be exported.
         * @param sSeparator Specifies the string that is used to separate multiple
         * field names (defaults to ` / `).
         * @returns Returns an array with fields.
         */
        function fieldsDiff(
            pInstanceOrContext: Instance | Context,
            pSelection?: string[],
            sSeparator?: string
        ): IFields;
        /**
         * Retrieves the data in CSV format.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pSelection Specifies the identifiers of the slots that should be exported.
         * @param sSeparator Specifies the string that is used to separate multiple
         * field names (defaults to ` / `).
         * @returns Returns a CSV object with the fields and the record.
         */
        function CSV(
            pInstanceOrContext: Instance | Context,
            pSelection?: string[],
            sSeparator?: string
        ): ICSV;
        /**
         * Retrieves the filled fields in CSV format.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pSelection Specifies the identifiers of the slots that should be exported.
         * @param sSeparator Specifies the string that is used to separate multiple
         * field names (defaults to ` / `).
         * @returns Returns a CSV object with the fields and the record.
         */
        function CSVFilled(
            pInstanceOrContext: Instance | Context,
            pSelection?: string[],
            sSeparator?: string
        ): ICSV;
        /**
         *
         * Retrieves the diff data in CSV format. Only fields that have changed are
         * included.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pSelection Specifies the identifiers of the slots that should be exported.
         * @param sSeparator Specifies the string that is used to separate multiple
         * field names (defaults to ` / `).
         * @returns Returns a CSV object with the fields and the record.
         */
        function CSVDiff(
            pInstanceOrContext: Instance | Context,
            pSelection?: string[],
            sSeparator?: string
        ): ICSV;
    }
}

declare module "tripetto-collector/lib/decorators/block" {
    import { IConditionBlockDecorator } from "tripetto-collector/lib/conditions/blocks/decorator";
    import {
        IHeadlessBlockDecorator,
        INodeBlockDecorator
    } from "tripetto-collector/lib/nodes/blocks/decorator";
    import { TConditionBlock } from "tripetto-collector/lib/conditions/blocks/type";
    import { TNodeBlock } from "tripetto-collector/lib/nodes/blocks/type";
    /**
     * Defines a node block.
     * @param pDecorator Specifies the settings for the block.
     */
    export function tripetto(
        pDecorator: INodeBlockDecorator | IHeadlessBlockDecorator
    ): (pBlock: TNodeBlock) => void;
    /**
     * Defines a condition block.
     * @param pDecorator Specifies the settings for the block.
     */
    export function tripetto(
        pDecorator: IConditionBlockDecorator
    ): (pBlock: TConditionBlock) => void;
}

declare module "tripetto-collector/lib/decorators/condition" {
    /** Dependencies */
    import { Callback } from "@marblecore/callback";
    import { ConditionBlock } from "tripetto-collector/lib/conditions/blocks/block";
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    export interface IConditionHandler {
        /**
         * Reference to the callback pointer. Pass this pointer as return value of
         * your condition function to enable asynchronous condition callback.
         */
        callback: Callback<boolean>;
    }
    export type ConditionResult = boolean | Callback<boolean>;
    /**
     * Marks the condition function that is invoked when the block is evaluated.
     * The function receives a `IConditionHandler` object.
     * The function should return a value of `ConditionResult`.
     */
    export function condition<T extends NodeBlock | ConditionBlock>(
        pTarget: T,
        sFunction: keyof T,
        pDescriptor?: PropertyDescriptor
    ): void;
}

declare module "tripetto-collector/lib/decorators/validate" {
    /** Dependencies */
    import { Callback } from "@marblecore/callback";
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    export interface IValidatorHandler {
        /**
         * Reference to the callback pointer. Pass this pointer as return value to
         * enable the asynchronous validation callback.
         */
        callback: Callback<boolean>;
        /** Contains the current validation state. */
        current: "unknown" | "fail" | "pass";
    }
    export type ValidatorResult = boolean | Callback<boolean>;
    /**
     * Marks the validation function that is invoked when the block is validated.
     * The function receives a `IValidatorHandler` object.
     * The function should return a value of type `ValidatorResult`.
     */
    export function validator<T extends NodeBlock>(
        pTarget: T,
        sFunction: keyof T,
        pDescriptor?: PropertyDescriptor
    ): void;
}

declare module "tripetto-collector/lib/decorators/destroy" {
    import { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    export interface IDestroyHandler {
        /** Specifies the reason for the destroy. */
        type: "left" | "canceled";
    }
    /**
     * Marks the destroy function that is invoked when the block is destroyed.
     * The function receives a `IDestroyHandler` object.
     */
    export function destroy<T extends NodeBlock>(
        pTarget: T,
        sFunction: keyof T,
        pDescriptor?: PropertyDescriptor
    ): void;
}

declare module "tripetto-collector/lib/contracts/styles" {
    export interface IStyleDependency {
        /** Specifies the property to depend on. */
        readonly property: string;
        /** Specifies the value that should be evaluated. */
        readonly value: string | number | boolean | undefined;
        /** Specifies the action to perform. */
        readonly action: "enable" | "disable" | "show" | "hide";
    }
    export type TStyleTiers = "premium";
    export interface IStyleStatic<Type extends string> {
        /** Specifies the style element type. */
        readonly type: Type;
        /** Specifies the variable and its value where the state of this style element depends on. */
        readonly dependency?: IStyleDependency;
        /** Specifies a label for the style element. */
        readonly label: string;
        /** Specifies a optional description for the style element. */
        readonly description?: string;
        /** Specifies the required tier for using this style element. */
        readonly tier?: TStyleTiers;
    }
    export interface IStyleElement<Type extends string, T>
        extends IStyleStatic<Type> {
        /** Specifies the name of the style element property. */
        readonly name: string;
        /** Specifies the default value for the style element. */
        readonly default?: T;
    }
    export type TStyleDropdownOption =
        | {
              /** Label for the option. */
              readonly label: string;
              /** Value for the option. */
              readonly value: string | number;
          }
        | {
              /** Specifies an option group. */
              readonly optGroup: string;
          }
        | {
              /** Specifies a custom option that allows the user to specify a custom value. */
              readonly custom: string;
          };
    export interface IStyleDropdown extends IStyleElement<"dropdown", string> {
        /** Specifies the dropdown options. */
        readonly options: TStyleDropdownOption[];
    }
    export interface IStyleRadiobutton {
        /** Label for the button. */
        readonly label: string;
        /** Value for the button. */
        readonly value: string | number;
        /** Optional description for the button. */
        readonly description?: string;
    }
    export interface IStyleRadiobuttons
        extends IStyleElement<"radiobuttons", string> {
        /** Specifies the radio buttons. */
        readonly buttons: IStyleRadiobutton[];
    }
    export type TStyleElement =
        | IStyleStatic<"static">
        | IStyleElement<"label" | "color" | "image", string>
        | IStyleElement<"checkbox", boolean>
        | IStyleDropdown
        | IStyleRadiobuttons;
    export interface IStyleCollection {
        /** Specifies the title for the collection of style elements. */
        readonly title: string;
        /** Specifies a description for the collection of style elements. */
        readonly description?: string;
        /** Specifies if the collection is optional (a checkbox will be shown before the collection to enable or disable it). */
        readonly optional?: boolean;
        /** Specifies a property name for the styles. */
        readonly name?: string;
        /** Specifies the variable and its value where the state of this collection depends on. */
        readonly dependency?: IStyleDependency;
        /** Specifies the required tier for using this collection. */
        readonly tier?: TStyleTiers;
        /** Specifies the style elements in this collection. */
        readonly elements: TStyleElement[];
    }
    export type TStylesContract = IStyleCollection[];
    export interface IStyles {
        [property: string]: string | number | boolean | IStyles | undefined;
    }
}

declare module "tripetto-collector/lib/contracts/l10n" {
    import { TTranslation } from "tripetto-collector/";
    export interface IL10nContract {
        /** Specifies the language identifiers of the available translations. */
        readonly availableTranslations?: string[];
        /** Specifies if the locale can be changed. */
        readonly customLocale?: boolean;
        /** Contains the translatable strings table. */
        readonly strings?: (string | [string, string])[];
    }
    export interface IL10n {
        /** Specifies the language of the translation to use. */
        readonly language?: "auto" | string;
        /** Specifies the locale to use. */
        readonly locale?: "auto" | string;
        /** Contains the translations. */
        readonly translations?: TTranslation | TTranslation[];
    }
}

declare module "tripetto-collector/lib/contracts/themes" {
    import { IStyles } from "tripetto-collector/lib/contracts/styles";
    export interface IThemeContract<T extends IStyles = IStyles> {
        /** Contains the theme name. */
        readonly name: string;
        /** Contains an optional description for the theme. */
        readonly description?: string;
        /** Contains a preview image (as URL or base64 encoded data). */
        readonly preview?: string;
        /** Contains the styles for the theme. */
        readonly styles: T;
    }
    export type TThemesContract<T extends IStyles = IStyles> = IThemeContract<
        T
    >[];
}

declare module "tripetto-collector/lib/storyline/orchestrator" {
    /** Dependencies */
    import { Instance } from "@tripetto/vfsm";
    import { TModes } from "tripetto-collector/lib/modes";
    export interface IStorylineOrchestrator {
        /** Instance of the storyline. */
        readonly instance: Instance;
        /** Contains the mode of operation. */
        readonly mode: TModes;
        /** Retrieves the checksum. */
        readonly checksum: string | undefined;
        /** Retrieves the direction. */
        readonly direction: "forward" | "backward";
    }
}

declare module "tripetto-collector/lib/conditions/blocks/decorator" {
    export interface IConditionBlockDecorator {
        /** Specifies this block is a condition block. */
        readonly type: "condition";
        /** Specifies the block identifier. */
        readonly identifier: string;
        /** Specifies optional type aliases. */
        readonly alias?: string | string[];
    }
}

declare module "tripetto-collector/lib/nodes/blocks/decorator" {
    export interface INodeBlockDecorator {
        /** Specifies this block is a node block. */
        readonly type: "node";
        /** Specifies the type identifier. */
        readonly identifier: string;
        /** Specifies optional type aliases. */
        readonly alias?: string | string[];
        /** Optional reference to something. */
        readonly ref?: {};
        /** Specifies if the block needs to be rerendered upon value store (disabled by default). */
        readonly autoRender?: boolean;
        /** Specifies if the block checks if all required slots contain a value (enabled by default). */
        readonly autoValidate?: boolean;
    }
    export interface IHeadlessBlockDecorator {
        /** Specifies this block is a headless block. */
        readonly type: "headless";
        /** Specifies the type identifier. */
        readonly identifier: string;
        /** Specifies optional type aliases. */
        readonly alias?: string | string[];
    }
}

declare module "tripetto-collector/" {
    /** Dependencies */
    import * as NSSlots from "@tripetto/slots";
    /** Contains the package name. */
    export const NAME: string;
    /** Contains the version number. */
    export const VERSION: string;
    /** Explicit exports */
    export {
        Context,
        Data,
        Errors,
        IDataValues,
        IMoment,
        Instance,
        ISnapshot,
        IVariable,
        Value,
        IOntologyResumeEvent as ICollectorResumeEvent
    } from "@tripetto/vfsm";
    export {
        ICondition,
        IConditionBlock,
        IDefinition,
        INode,
        INodeBlock
    } from "@tripetto/map";
    export { Kinds, Slot } from "@tripetto/slots";
    export { Condition } from "tripetto-collector/lib/conditions/condition";
    export {
        ConditionBlock
    } from "tripetto-collector/lib/conditions/blocks/block";
    export {
        ConditionBlocks
    } from "tripetto-collector/lib/conditions/blocks/blocks";
    export {
        IConditionBlockType,
        TConditionBlock
    } from "tripetto-collector/lib/conditions/blocks/type";
    export { Branch } from "tripetto-collector/lib/branches/branch";
    export { Cluster } from "tripetto-collector/lib/clusters/cluster";
    export { Collector } from "tripetto-collector/lib/collector";
    export { Node } from "tripetto-collector/lib/nodes/node";
    export { NodeBlock } from "tripetto-collector/lib/nodes/blocks/block";
    export {
        HeadlessBlock
    } from "tripetto-collector/lib/nodes/blocks/headless";
    export { NodeBlocks } from "tripetto-collector/lib/nodes/blocks/blocks";
    export {
        INodeBlockType,
        TNodeBlock
    } from "tripetto-collector/lib/nodes/blocks/type";
    export {
        IObservableNode
    } from "tripetto-collector/lib/storyline/observable";
    export {
        ICollectorChangeEvent,
        ICollectorRestartEvent
    } from "tripetto-collector/lib/events";
    export { TChanges } from "tripetto-collector/lib/changes";
    export { TModes } from "tripetto-collector/lib/modes";
    export { TStatus } from "tripetto-collector/lib/status";
    export {
        markdownify,
        markdownifyTo,
        markdownifyToString
    } from "tripetto-collector/lib/markdown";
    export { MarkdownFeatures, MarkdownTypes } from "@marblecore/markdown";
    export { Storyline } from "tripetto-collector/lib/storyline/storyline";
    export { IStoryline } from "tripetto-collector/lib/storyline/interface";
    export { Moment } from "tripetto-collector/lib/storyline/moment";
    export { IPage } from "tripetto-collector/lib/storyline/page";
    export { Import } from "tripetto-collector/lib/data/import";
    export { Export } from "tripetto-collector/lib/data/export";
    /** Decorators */
    export {
        tripetto,
        tripetto as block
    } from "tripetto-collector/lib/decorators/block";
    export {
        condition,
        IConditionHandler,
        ConditionResult
    } from "tripetto-collector/lib/decorators/condition";
    export {
        validator,
        IValidatorHandler,
        ValidatorResult
    } from "tripetto-collector/lib/decorators/validate";
    export {
        destroy,
        IDestroyHandler
    } from "tripetto-collector/lib/decorators/destroy";
    /** Slots */
    export import Slots = NSSlots;
    /** Implicit exports */
    export * from "@marblecore/callback";
    export * from "@marblecore/functional";
    export * from "@marblecore/sha2";
    export * from "@marblecore/l10n";
    /** Contracts */
    export * from "tripetto-collector/lib/contracts/styles";
    export * from "tripetto-collector/lib/contracts/l10n";
    export * from "tripetto-collector/lib/contracts/themes";
}
