/*! Tripetto Collector 1.7.1-29 - Copyright (C) 2019 Tripetto B.V. - All Rights Reserved */

/// <reference path="marblecore.functional.d.ts" />
/// <reference path="marblecore.grid.d.ts" />
/// <reference path="marblecore.matrix.d.ts" />
/// <reference path="marblecore.metrics.d.ts" />
/// <reference path="marblecore.markdown.d.ts" />
/// <reference path="marblecore.serializer.d.ts" />
/// <reference path="tripetto.slots.d.ts" />
/// <reference path="marblecore.hookup.d.ts" />

declare module "@tripetto/map" {
    /** Explicit exports */
    export { Branch } from "@tripetto/map/lib/branches/branch";
    export { Branches } from "@tripetto/map/lib/branches/branches";
    export { Cluster } from "@tripetto/map/lib/clusters/cluster";
    export { Clusters } from "@tripetto/map/lib/clusters/clusters";
    export { Condition } from "@tripetto/map/lib/conditions/condition";
    export { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    export { Conditions } from "@tripetto/map/lib/conditions/conditions";
    export { IBranch } from "@tripetto/map/lib/branches/interface";
    export { ICluster } from "@tripetto/map/lib/clusters/interface";
    export { ICondition } from "@tripetto/map/lib/conditions/interface";
    export {
        IConditionBlock
    } from "@tripetto/map/lib/conditions/blocks/interface";
    export {
        IConditionBlockDecorator
    } from "@tripetto/map/lib/conditions/blocks/decorator";
    export {
        IConditionBlockType,
        TConditionBlock
    } from "@tripetto/map/lib/conditions/blocks/type";
    export { IDefinition } from "@tripetto/map/lib/interface";
    export { IEditor } from "@tripetto/map/lib/editor";
    export { INode } from "@tripetto/map/lib/nodes/interface";
    export { INodeBlock } from "@tripetto/map/lib/nodes/blocks/interface";
    export {
        INodeBlockDecorator
    } from "@tripetto/map/lib/nodes/blocks/decorator";
    export {
        INodeBlockType,
        TNodeBlock
    } from "@tripetto/map/lib/nodes/blocks/type";
    export { Map } from "@tripetto/map/lib/map";
    export { Node } from "@tripetto/map/lib/nodes/node";
    export { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    export { Nodes } from "@tripetto/map/lib/nodes/nodes";
    export { TBranchCulling } from "@tripetto/map/lib/branches/culling";
    export { TBranchTerminator } from "@tripetto/map/lib/branches/terminator";
    export { IVariable } from "@tripetto/map/lib/variables";
    export { observe as detectChange } from "@tripetto/map/lib/observer";
    export { isRequired } from "@tripetto/map/lib/decorators/affects/required";
    export {
        BlocksNamespaces,
        IBlocksNamespace
    } from "@tripetto/map/lib/namespaces";
    /** Decorators */
    export { affects } from "@tripetto/map/lib/decorators/affects";
    export { assigned } from "@tripetto/map/lib/decorators/assigned";
    export { block } from "@tripetto/map/lib/decorators/block";
    export { created } from "@tripetto/map/lib/decorators/created";
    export { definition } from "@tripetto/map/lib/decorators/definition";
    export { destroyed } from "@tripetto/map/lib/decorators/destroyed";
    export { detached } from "@tripetto/map/lib/decorators/detached";
    export { slots } from "@tripetto/map/lib/decorators/slots";
    export { unassigned } from "@tripetto/map/lib/decorators/unassigned";
    export { upgraded } from "@tripetto/map/lib/decorators/upgraded";
    /** Events */
    export { IMapChangeEvent, IMapErrorEvent } from "@tripetto/map/lib/events";
    export {
        IBranchCullingEvent,
        IBranchTerminatorEvent
    } from "@tripetto/map/lib/branches/events";
    export {
        IConditionBlockAssignEvent,
        IConditionBlockChangeEvent,
        IConditionBlockUnassignEvent
    } from "@tripetto/map/lib/conditions/events";
    export {
        INodeBlockAssignEvent,
        INodeBlockChangeEvent,
        INodeBlockUnassignEvent
    } from "@tripetto/map/lib/nodes/events";
}

declare module "@tripetto/map/lib/branches/branch" {
    /** Dependencies */
    import { TBranchEvents } from "@tripetto/map/lib/branches/events";
    import { IBranch } from "@tripetto/map/lib/branches/interface";
    import { Clusters } from "@tripetto/map/lib/clusters/clusters";
    import { Condition } from "@tripetto/map/lib/conditions/condition";
    import { Conditions } from "@tripetto/map/lib/conditions/conditions";
    import { Stack } from "@marblecore/functional";
    import { GridBranch, IGridResource } from "@marblecore/grid";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { Branches } from "@tripetto/map/lib/branches/branches";
    import { IItemChangeEvent } from "@marblecore/matrix";
    import { IPoint } from "@marblecore/metrics";
    import { Map } from "@tripetto/map/lib/map";
    import { Node } from "@tripetto/map/lib/nodes/node";
    import { TBranchCulling } from "@tripetto/map/lib/branches/culling";
    import { TBranchTerminator } from "@tripetto/map/lib/branches/terminator";
    export class Branch extends GridBranch<Clusters, Node, TBranchEvents> {
        /** Retrieves if the branch is validating. */
        static readonly isValidating: boolean;
        /**
         * Creates a new branch instance.
         * @param pBranches Reference to the branch dispatcher.
         */
        constructor(pBranches: Branches);
        /** Retrieves the identifier of the branch. */
        readonly id: string;
        /** Retrieves the parent branch dispatcher. */
        readonly parent: Branches | undefined;
        /** Retrieves the predecessor of the branch. */
        readonly predecessor: Branch | undefined;
        /** Retrieves the successor of the branch. */
        readonly successor: Branch | undefined;
        /** Retrieves the parent map. */
        readonly map: Map | undefined;
        /** Retrieves the parent cluster. */
        readonly cluster: Cluster | undefined;
        /** Retrieves the conditions. */
        readonly conditions: Conditions;
        /** Retrieves the clusters. */
        readonly clusters: Clusters;
        /** Retrieves the branch spacing. */
        readonly spacing: number;
        /** Sets the offset for the branch. */
        offset: number;
        /** Sets the culling mode. */
        culling: TBranchCulling;
        /** Sets the termination mode for the branch. */
        terminator: TBranchTerminator;
        /** Sets the jump-to-cluster. */
        jump: Cluster | undefined;
        /** Sets the name of the branch. */
        name: string | undefined;
        /**
         * Fired when a branch is changed.
         * @param pEvent Contains the event payload.
         * @event
         */
        protected onChange(pEvent: IItemChangeEvent<this>): void;
        /**
         * Fired when a branch move occurs.
         * @param nMoveX Specifies the horizontal movement.
         * @param nMoveY Specifies the vertical movement.
         * @param pOffset Specifies the offset of the touch point.
         * @event
         */
        onMove(nMoveX: number, nMoveY: number, pOffset: IPoint): void;
        /**
         * Iterate through each cluster and condition of the branch.
         * @param fnResource Specifies the resource function.
         * @param fnResource.pResource Contains a reference to the resource.
         * @return Returns `true` if the iteration is stopped (break).
         */
        forEachDescendant(
            fnResource: (pResource: Condition | Cluster) => boolean | void
        ): boolean;
        /**
         * Applies the spacing for the conditions.
         * @param nWidth Specifies the width of the resource.
         * @param nHeight Specifies the height of the resource.
         * @return Returns `true` if the resource is resized.
         */
        resize(nWidth?: number, nHeight?: number): boolean;
        /**
         * Generates a serialized representation of a branch.
         * @return Returns a reference to the serialized branch.
         */
        serialize(): IBranch;
        /**
         * Deserializes to a branch.
         * @param pBranch Contains the serialized data.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @return Returns a reference to the instance.
         */
        deserialize(
            pBranch: IBranch & IGridResource,
            pAfterwards?: Stack
        ): this;
        /**
         * Inserts a new branch before this branch.
         * @return Returns a reference to the new branch.
         */
        insertBefore(): Branch;
        /**
         * Inserts a new branch after this branch.
         * @return Returns a reference to the new branch.
         */
        insertAfter(): Branch;
        /**
         * Duplicates the branch.
         * @param pDest Specifies the destination branch. If omitted a new branch is inserted after this branch.
         * @return Returns a reference to the new branch.
         */
        duplicate(pDest?: Branch): Branch;
        /** Deletes this branch from the map. */
        delete(): this;
        /** Resets the branch. */
        reset(): void;
        /** Destroys a branch. */
        destroy(): void;
    }
}

declare module "@tripetto/map/lib/branches/branches" {
    /** Dependencies */
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { Stack } from "@marblecore/functional";
    import { GridBranches, IGridResource } from "@marblecore/grid";
    import { IBranch } from "@tripetto/map/lib/branches/interface";
    import { IBranchConstructor } from "@tripetto/map/lib/branches/constructor";
    import { Node } from "@tripetto/map/lib/nodes/node";
    export class Branches extends GridBranches<Branch, Node> {
        constructor(pBranchConstructor: IBranchConstructor, pCluster: Cluster);
        /** Retrieves the cluster of the branch dispatcher. */
        readonly cluster: Cluster;
        /**
         * Invoked when the branch dimensions are changed. This aligns the space
         * occupied by the conditions.
         * @event
         */
        onDimensions(): void;
        /**
         * Appends a new branch.
         * @return Returns the new branch.
         */
        append(): Branch;
        /**
         * Inserts a new branch.
         * @return Returns the new branch.
         */
        insert(): Branch;
        /**
         * Inserts a new branch before the supplied branch.
         * @param pBranch Specifies the reference branch.
         * @return Returns the new branch.
         */
        insertBefore(pBranch: Branch): Branch;
        /**
         * Inserts a new branch after the supplied branch.
         * @param pBranch Specifies the reference branch.
         * @return Returns the new branch.
         */
        insertAfter(pBranch: Branch): Branch;
        /**
         * Deserializes the supplied serialized branches to the branch dispatcher.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @param pBranches Specifies the branches.
         * @return Returns a reference to the instance.
         */
        deserialize(
            pAfterwards: Stack,
            ...pBranches: (IBranch & IGridResource)[]
        ): this;
    }
}

declare module "@tripetto/map/lib/clusters/cluster" {
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { Clusters } from "@tripetto/map/lib/clusters/clusters";
    import { Stack } from "@marblecore/functional";
    import { GridCluster } from "@marblecore/grid";
    import { Branches } from "@tripetto/map/lib/branches/branches";
    import { ICluster } from "@tripetto/map/lib/clusters/interface";
    import { IItemChangeEvent, IItemPopEvent } from "@marblecore/matrix";
    import { Map } from "@tripetto/map/lib/map";
    import { Node } from "@tripetto/map/lib/nodes/node";
    import { Nodes } from "@tripetto/map/lib/nodes/nodes";
    import { TClusterEvents } from "@tripetto/map/lib/clusters/events";
    export class Cluster extends GridCluster<Nodes, Node, TClusterEvents> {
        /**
         * Creates a new cluster instance.
         * @param pClusters Reference to the cluster dispatcher.
         */
        constructor(pClusters: Clusters);
        /** Retrieves the identifier of the cluster. */
        readonly id: string;
        /** Retrieves the parent cluster dispatcher. */
        readonly parent: Clusters | undefined;
        /** Retrieves the predecessor of the cluster. */
        readonly predecessor: Cluster | undefined;
        /** Retrieves the successor of the cluster. */
        readonly successor: Cluster | undefined;
        /** Retrieves the parent map. */
        readonly map: Map | undefined;
        /** Retrieves the parent branch. */
        readonly branch: Branch | undefined;
        /** Retrieves the nodes. */
        readonly nodes: Nodes;
        /** Retrieves the branches. */
        readonly branches: Branches;
        /** Retrieves if the cluster has branches. */
        readonly hasBranches: boolean;
        /** Sets the name of the cluster. */
        name: string | undefined;
        /** Retrieves a list of branches which have a jump to this cluster. */
        readonly jumps: Branch[];
        /**
         * Fired when a cluster is changed.
         * @param pEvent Contains the event payload.
         * @event
         */
        protected onChange(pEvent: IItemChangeEvent<this>): void;
        /**
         * Invoked when the cluster is popped.
         * @param pEvent Contains the event payload.
         * @event
         */
        onPop(pEvent: IItemPopEvent<this>): void;
        /**
         * Iterate through each node and branch of the cluster.
         * @param fnResource Specifies the resource function.
         * @param fnResource.pResource Contains a reference to the resource.
         * @return Returns `true` if the iteration is stopped (break).
         */
        forEachDescendant(
            fnResource: (pResource: Node | Branch) => boolean | void
        ): boolean;
        /**
         * Calculates the dimensions of the cluster (only necessary when we are not moving).
         * @param nWidth Specifies the width of the cluster.
         * @param nHeight Specifies the height of the cluster.
         * @return Returns `true` if the cluster is resized.
         */
        resize(nWidth?: number, nHeight?: number): boolean;
        /**
         * Generates a serialized representation of a cluster.
         * @return Returns a reference to the serialized cluster.
         */
        serialize(): ICluster;
        /**
         * Deserializes to a cluster.
         * @param pCluster Contains the serialized cluster.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @return Returns a reference to the instance.
         */
        deserialize(pCluster: ICluster, pAfterwards?: Stack): this;
        /**
         * Inserts a new cluster before this cluster.
         * @return Returns a reference to the new cluster.
         */
        insertBefore(): Cluster;
        /**
         * Inserts a new cluster after this cluster.
         * @return Returns a reference to the new cluster.
         */
        insertAfter(): Cluster;
        /**
         * Duplicates the cluster.
         * @param pDest Specifies the destination cluster. If omitted a new cluster is inserted after this cluster.
         * @return Returns a reference to the new cluster.
         */
        duplicate(pDest?: Cluster): Cluster;
        /** Deletes this cluster from the map. */
        delete(): this;
        /** Resets the cluster. */
        reset(): void;
        /** Destroys a cluster. */
        destroy(): void;
    }
}

declare module "@tripetto/map/lib/clusters/clusters" {
    /** Dependencies */
    import { Map } from "@tripetto/map/lib/map";
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { Stack } from "@marblecore/functional";
    import { GridClusters } from "@marblecore/grid";
    import { ICluster } from "@tripetto/map/lib/clusters/interface";
    import { IClusterConstructor } from "@tripetto/map/lib/clusters/constructor";
    import { Node } from "@tripetto/map/lib/nodes/node";
    export class Clusters extends GridClusters<Cluster, Node> {
        constructor(pClusterConstructor: IClusterConstructor, pBranch: Branch);
        /** Retrieves the map of the cluster dispatcher. */
        readonly map: Map | undefined;
        /** Retrieves the branch of the cluster dispatcher. */
        readonly branch: Branch | Map | undefined;
        /**
         * Invoked when the cluster dimensions are changed.
         * @event
         */
        onDimensions(): void;
        /**
         * Appends a new cluster.
         * @return Returns the new cluster.
         */
        append(): Cluster;
        /**
         * Inserts a new cluster.
         * @return Returns the new cluster.
         */
        insert(): Cluster;
        /**
         * Inserts a new cluster before the supplied cluster.
         * @param pCluster Specifies the reference cluster.
         * @return Returns the new cluster.
         */
        insertBefore(pCluster: Cluster): Cluster;
        /**
         * Inserts a new cluster after the supplied cluster.
         * @param pCluster Specifies the reference cluster.
         * @return Returns the new branch.
         */
        insertAfter(pCluster: Cluster): Cluster;
        /**
         * Deserializes the supplied serialized clusters to the cluster dispatcher.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @param pClusters Specifies the clusters.
         * @return Returns a reference to the instance.
         */
        deserialize(pAfterwards: Stack, ...pClusters: ICluster[]): this;
    }
}

declare module "@tripetto/map/lib/conditions/condition" {
    /** Dependencies */
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    import { Conditions } from "@tripetto/map/lib/conditions/conditions";
    import { GridNode } from "@marblecore/grid";
    import { ICondition } from "@tripetto/map/lib/conditions/interface";
    import { TConditionEvents } from "@tripetto/map/lib/conditions/events";
    import { IConditionBlockType } from "@tripetto/map/lib/conditions/blocks/type";
    import { IItemChangeEvent } from "@marblecore/matrix";
    import { IPoint } from "@marblecore/metrics";
    import { IVariable } from "@tripetto/map/lib/variables";
    import { Map } from "@tripetto/map/lib/map";
    import { MarkdownParser } from "@marblecore/markdown";
    import { Stack } from "@marblecore/functional";
    export class Condition extends GridNode<TConditionEvents> {
        /**
         * Creates a new condition instance.
         * @param pConditions Reference to the condition dispatcher.
         */
        constructor(pConditions: Conditions);
        /** Retrieves the identifier of the condition. */
        readonly id: string;
        /** Retrieves the parent condition dispatcher. */
        readonly parent: Conditions | undefined;
        /** Retrieves the predecessor of the condition. */
        readonly predecessor: Condition | undefined;
        /** Retrieves the successor of the condition. */
        readonly successor: Condition | undefined;
        /** Retrieves the parent map. */
        readonly map: Map | undefined;
        /** Retrieves the collection. */
        readonly branch: Branch | undefined;
        /** Retrieves the parent cluster of the condition. */
        readonly cluster: Cluster | undefined;
        /** Retrieves the block for the condition. */
        readonly block: ConditionBlock | undefined;
        /** Retrieves the block type identifier as a string. */
        readonly blockTypeIdentifier: string;
        /** Retrieves if the condition has a block. */
        readonly hasBlock: boolean;
        /** Retrieves if the block is invalid. */
        readonly isBlockInvalid: boolean;
        /** Specifies if the condition is disabled. */
        disabled: boolean | undefined;
        /** Retrieves the condition type. */
        readonly type: IConditionBlockType<ConditionBlock> | undefined;
        /** Retrieves the condition name. */
        readonly name: string;
        /** Retrieves a markdown instance for the name. */
        readonly nameMarkdown: MarkdownParser<IVariable>;
        /** Retrieves the condition label. */
        readonly label: string;
        /**
         * Fired when the condition is changed.
         * @param pEvent Specifies the event payload.
         * @event
         */
        protected onChange(pEvent: IItemChangeEvent<this>): void;
        /**
         * Fired when a condition move occurs.
         * @param nMoveX Specifies the horizontal movement.
         * @param nMoveY Specifies the vertical movement.
         * @param pOffset Specifies the offset of the touch point.
         * @event
         */
        onMove(nMoveX: number, nMoveY: number, pOffset: IPoint): void;
        /**
         * Invokes the condition layout renderer.
         * @param sType Specifies the rerender type. Can be one of the following values:
         * - `refresh`: Refresh the rendering;
         * - `update`: Update the rendering.
         */
        rerender(sType: "refresh" | "update"): void;
        /**
         * Assigns a block to the condition.
         * @param pBlock Specifies the block type.
         * @return Returns a reference to the block instance.
         */
        assignBlock<T extends ConditionBlock>(
            pBlock: IConditionBlockType<T> | T
        ): T | undefined;
        /**
         * Unassigns a block.
         * @return Returns the previously assigned block or `undefined` if there
         * was none.
         */
        unassignBlock(): ConditionBlock | undefined;
        /**
         * Generates a serialized representation of a condition.
         * @return Returns a reference to the serialized condition.
         */
        serialize(): ICondition;
        /**
         * Deserializes to a condition.
         * @param pCondition Contains the serialized condition.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @return Returns a reference to the instance.
         */
        deserialize(pCondition: ICondition, pAfterwards?: Stack): this;
        /**
         * Inserts a new condition before this condition.
         * @return Returns a reference to the new condition.
         */
        insertBefore(): Condition;
        /**
         * Inserts a new condition after this condition.
         * @return Returns a reference to the new condition.
         */
        insertAfter(): Condition;
        /** Enables the condition. */
        enable(): this;
        /** Disables the condition. */
        disable(): this;
        /**
         * Duplicates the condition.
         * @param pDest Specifies the destination condition. If omitted a new condition is inserted after this condition.
         * @return Returns a reference to the new condition.
         */
        duplicate(pDest?: Condition): Condition;
        /** Deletes this condition from the map. */
        delete(): this;
        /** Destroys the condition. */
        destroy(): void;
    }
}

declare module "@tripetto/map/lib/conditions/blocks/block" {
    /** Dependencies */
    import { IConditionBlockType } from "@tripetto/map/lib/conditions/blocks/type";
    import { Stack } from "@marblecore/functional";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { Condition } from "@tripetto/map/lib/conditions/condition";
    import { ISerialize } from "@marblecore/serializer";
    import { IConditionBlock } from "@tripetto/map/lib/conditions/blocks/interface";
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { Labels } from "@tripetto/map/lib/conditions/blocks/labels";
    import { Map } from "@tripetto/map/lib/map";
    import { Node } from "@tripetto/map/lib/nodes/node";
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    import { Slot } from "@tripetto/slots";
    import { TConditionBlockContext } from "@tripetto/map/lib/conditions/blocks/context";
    export abstract class ConditionBlock {
        /** Contains the type identifier of the block. */
        static readonly identifier: string;
        /** Contains the version of the block. */
        static readonly version: string;
        /** Contains the block context. */
        static readonly context: TConditionBlockContext;
        /** Contains the label localizer. */
        static readonly _: () => string;
        /** Retrieves the localized label (name) of the block. */
        static readonly label: string;
        /** Retrieves the block type. */
        readonly type: IConditionBlockType<ConditionBlock>;
        /** Contains the branch. */
        branch?: Branch;
        /** Contains the cluster. */
        cluster?: Cluster;
        /** Contains the node. */
        node?: Node;
        /** Contains the slot. */
        slot?: Slot;
        /**
         * Creates a new block instance or verifies an existing instance.
         * @param pBlock Reference to the block constructor or instance.
         * @param pCondition Reference to the parent condition.
         * @param bInitialize Specifies if the block should be initialized.
         * @return Returns the block instance.
         */
        static create<Block extends ConditionBlock>(
            pBlock: IConditionBlockType<Block> | Block | string,
            pCondition: Condition,
            bInitialize: boolean
        ): Block | undefined;
        /**
         * Constructs a new block.
         * @param pCondition Reference to the condition.
         */
        constructor(pCondition: Condition);
        /** Retrieves the identifier of the condition. */
        readonly id: string;
        /** Retrieves the parent map. */
        readonly map: Map | undefined;
        /** Retrieves a reference to the condition. */
        readonly condition: Condition;
        /** Retrieves if the block is initialized. */
        readonly isInitialized: boolean;
        /** Retrieves the name of the condition block. */
        readonly name: string;
        /**
         * Retrieves the labels collection. This is a collection of labels that give
         * context to the condition.
         */
        readonly labels: Labels;
        /**
         * Retrieves the label of the condition block. This label contains additional
         * context for the condition. For example, if a node is attached to the
         * condition, the label will contain the name of that node. Otherwise, the
         * name of the parent cluster will be used or the name of the parent branch.
         */
        readonly label: string;
        /**
         * Block is assigned to a condition.
         * @param pPrevious Reference to the previous block that was attaced to the condition.
         * @return Returns a reference to the block instance.
         */
        assignedToCondition(pPrevious: ConditionBlock | undefined): this;
        /** Block is unassigned from the condition. */
        unassignedFromCondition(): void;
        /**
         * Invoked when the block of a node is unassigned. This may affect the
         * condition. This function verifies if the condition should be deleted.
         * @param pBlock Reference to the node block that is unassigned.
         * @return Should return `true` if the condition should be deleted.
         */
        shouldBeDeleted(pBlock: NodeBlock): boolean;
        /** Marks a change in the block. */
        detectChange(): void;
        /**
         * Rerenders the block.
         * @param bShouldRerender Specifies if the condition should rerender.
         * @event
         */
        rerender(bShouldRerender?: boolean): void;
        /**
         * Serializes a block.
         * @param pProperties Specifies additional serialized properties.
         * @return Returns a reference to the serialized block.
         */
        serialize(pProperties?: ISerialize): IConditionBlock;
        /**
         * Deserializes to a block instance.
         * @param pBlock Contains the serialized block.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @return Returns a reference to the instance.
         */
        deserialize(pBlock: IConditionBlock, pAfterwards: Stack): this;
        /**
         * Deletes the condition.
         * @param sProperty Optional property causing the removal.
         */
        delete(sProperty?: keyof this): void;
        /** Destroys the block. */
        destroy(): void;
    }
}

declare module "@tripetto/map/lib/conditions/conditions" {
    /** Dependencies */
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { Condition } from "@tripetto/map/lib/conditions/condition";
    import { Stack } from "@marblecore/functional";
    import { GridNodes } from "@marblecore/grid";
    import { ICondition } from "@tripetto/map/lib/conditions/interface";
    import { IConditionConstructor } from "@tripetto/map/lib/conditions/constructor";
    export class Conditions extends GridNodes<Condition> {
        constructor(
            pConditionConstructor: IConditionConstructor,
            pBranch: Branch
        );
        /** Retrieves the branch of the condition dispatcher. */
        readonly branch: Branch;
        /**
         * Invoked when the conditions dimensions are changed.
         * @event
         */
        onDimensions(): void;
        /**
         * Appends a new condition.
         * @return Returns the new condition.
         */
        append(): Condition;
        /**
         * Inserts a new condition.
         * @return Returns the new condition.
         */
        insert(): Condition;
        /**
         * Inserts a new condition before the supplied condition.
         * @param pCondition Specifies the reference condition.
         * @return Returns the new condition.
         */
        insertBefore(pCondition: Condition): Condition;
        /**
         * Inserts a new condition after the supplied condition.
         * @param pCondition Specifies the reference condition.
         * @return Returns the new condition.
         */
        insertAfter(pCondition: Condition): Condition;
        /**
         * Deserializes the supplied serialized conditions to the condition dispatcher.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @param pConditions Specifies the conditions.
         * @return Returns a reference to the instance.
         */
        deserialize(pAfterwards: Stack, ...pConditions: ICondition[]): this;
    }
}

declare module "@tripetto/map/lib/branches/interface" {
    /** Dependencies */
    import { ICluster } from "@tripetto/map/lib/clusters/interface";
    import { ICondition } from "@tripetto/map/lib/conditions/interface";
    import { IGridResource } from "@marblecore/grid";
    import { TBranchCulling } from "@tripetto/map/lib/branches/culling";
    export interface IBranch extends IGridResource {
        readonly conditions?: ICondition[];
        readonly clusters?: ICluster[];
        readonly culling?: TBranchCulling;
        readonly jump?: string;
        readonly name?: string;
    }
}

declare module "@tripetto/map/lib/clusters/interface" {
    /** Dependencies */
    import { IBranch } from "@tripetto/map/lib/branches/interface";
    import { IGridResource } from "@marblecore/grid";
    import { INode } from "@tripetto/map/lib/nodes/interface";
    export interface ICluster extends IGridResource {
        readonly branches?: IBranch[];
        readonly nodes?: INode[];
        readonly name?: string;
    }
}

declare module "@tripetto/map/lib/conditions/interface" {
    /** Dependencies */
    import { IConditionBlock } from "@tripetto/map/lib/conditions/blocks/interface";
    import { IGridResource } from "@marblecore/grid";
    export interface ICondition extends IGridResource {
        readonly block?: IConditionBlock;
        readonly disabled?: boolean;
    }
}

declare module "@tripetto/map/lib/conditions/blocks/interface" {
    /** Dependencies */
    import { ISerialize } from "@marblecore/serializer";
    export interface IConditionBlock extends ISerialize {
        /** Contains the condition block type. */
        readonly type: string;
        /** Contains the version of the block. */
        readonly version: string;
        /** Contains the id of an attached branch. */
        readonly branch?: string;
        /** Contains the id of an attached cluster. */
        readonly cluster?: string;
        /** Contains the id of an attached node. */
        readonly node?: string;
        /** Contains the id of an attached slot. */
        readonly slot?: string;
    }
}

declare module "@tripetto/map/lib/conditions/blocks/decorator" {
    /** Dependencies */
    import { TConditionBlockContext } from "@tripetto/map/lib/conditions/blocks/context";
    export interface IConditionBlockDecorator {
        /** Specifies this block is a condition block. */
        readonly type: "condition";
        /** Specifies the block identifier. */
        readonly identifier: string;
        /** Specifies optional type aliases. */
        readonly alias?: string | string[];
        /** Specifies the localized block label (name). If you use `gettext`, return the localized string using a function. */
        readonly label: string | (() => string);
        /**
         * Specifies the block context. Can be one of the following values:
         * - `*`: Conditions that can be used anywhere in the map;
         * - `cluster`: If the condition applies to clusters;
         * - `branch`: If the condition applies to branches;
         * - `node`: If the condition applies to nodes;
         * - A reference to a node block (the condition applies to nodes of that specific type);
         * - The type string identifier of a node block (the condition applies to nodes of that specific type).
         */
        readonly context: TConditionBlockContext | string;
        /** Specifies the version of the block in SemVer format (defaults to `0.0.0`). */
        readonly version?: string;
    }
}

declare module "@tripetto/map/lib/conditions/blocks/type" {
    /** Dependencies */
    import { Condition } from "@tripetto/map/lib/conditions/condition";
    import { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    import { TConditionBlockContext } from "@tripetto/map/lib/conditions/blocks/context";
    export interface IConditionBlockType<T extends ConditionBlock> {
        /** Contains the block type identifier. */
        readonly identifier: string;
        /** Contains the block label (name). */
        readonly label: string;
        /** Contains the block version. */
        readonly version: string;
        /** Contains the block context. */
        readonly context: TConditionBlockContext;
        /** Contains the label localizer. */
        readonly _: () => string;
        /**
         * Creates a new block instance.
         * @param pCondition Reference to the condition.
         */
        new (pCondition: Condition): T;
    }
    export type TConditionBlock<
        T extends ConditionBlock = ConditionBlock
    > = IConditionBlockType<T>;
}

declare module "@tripetto/map/lib/interface" {
    /** Dependencies */
    import { ICluster } from "@tripetto/map/lib/clusters/interface";
    import { IEditor } from "@tripetto/map/lib/editor";
    export interface IDefinition {
        /** Contains the clusters. */
        readonly clusters: ICluster[];
        /** Contains information abouw the editor. */
        readonly editor: IEditor;
        /** Contains the map name. */
        readonly name?: string;
        /** Contains the map description. */
        readonly description?: string;
        /** Contains the map keywords. */
        readonly keywords?: string[];
        /** Contains the language of the map. */
        readonly language?: string;
    }
}

declare module "@tripetto/map/lib/editor" {
    export interface IEditor {
        /** Editor name. */
        name: string;
        /** Editor version. */
        version: string;
    }
}

declare module "@tripetto/map/lib/nodes/interface" {
    /** Dependencies */
    import { IGridResource } from "@marblecore/grid";
    import { ISlot } from "@tripetto/slots";
    import { INodeBlock } from "@tripetto/map/lib/nodes/blocks/interface";
    export interface INode extends IGridResource {
        readonly block?: INodeBlock;
        readonly slots?: ISlot[];
        readonly disabled?: boolean;
        readonly name?: string;
        readonly nameVisible?: boolean;
        readonly placeholder?: string;
        readonly description?: string;
        readonly explanation?: string;
    }
}

declare module "@tripetto/map/lib/nodes/blocks/interface" {
    /** Dependencies */
    import { ISerialize } from "@marblecore/serializer";
    export interface INodeBlock extends ISerialize {
        /** Contains the node block type. */
        readonly type: string;
        /** Contains the version of the block. */
        readonly version: string;
    }
}

declare module "@tripetto/map/lib/nodes/blocks/decorator" {
    export interface INodeBlockDecorator {
        /** Specifies this block is a node block. */
        readonly type: "node";
        /** Specifies the type identifier. */
        readonly identifier: string;
        /** Specifies optional type aliases. */
        readonly alias?: string | string[];
        /** Specifies the localized block label (name). If you use `gettext`, return the localized string using a function. */
        readonly label: string | (() => string);
        /** Specifies the version of the block in SemVer format (defaults to `0.0.0`). */
        readonly version?: string;
        /**
         * Specifies the kind of block. Can be one of the following values:
         * - `headless`: Specifies the block is headless (needs no user interface);
         * - `ui`: Specifies the block needs a user interface.
         */
        readonly kind?: "headless" | "ui";
    }
}

declare module "@tripetto/map/lib/nodes/blocks/type" {
    /** Dependencies */
    import { Node } from "@tripetto/map/lib/nodes/node";
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    export interface INodeBlockType<T extends NodeBlock> {
        /** Contains the block type identifier. */
        readonly identifier: string;
        /** Contains the block label (name). */
        readonly label: string;
        /** Contains the block version. */
        readonly version: string;
        /** Contains the kind of block. */
        readonly kind: "headless" | "ui";
        /** Contains the label localizer. */
        readonly _: () => string;
        /**
         * Creates a new block instance.
         * @param pNode Reference to the node.
         */
        new (pNode: Node): T;
    }
    export type TNodeBlock<T extends NodeBlock = NodeBlock> = INodeBlockType<T>;
}

declare module "@tripetto/map/lib/map" {
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { Clusters } from "@tripetto/map/lib/clusters/clusters";
    import { Condition } from "@tripetto/map/lib/conditions/condition";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { Grid, GridRenderer, IGridResource } from "@marblecore/grid";
    import { IDefinition } from "@tripetto/map/lib/interface";
    import { IEditor } from "@tripetto/map/lib/editor";
    import { Node } from "@tripetto/map/lib/nodes/node";
    export class Map extends Grid<
        Clusters,
        Node,
        "OnRename" | "OnChange" | "OnError"
    > {
        /** Contains the name of the map. */
        name?: string;
        /** Contains the description of the map. */
        description?: string;
        /** Contains the language of the map. */
        language?: string;
        /** Contains the keywords for the map. */
        keywords?: string[];
        /**
         * Creates a new map instance.
         * @param pRenderer Reference to a renderer for the map.
         * @param pEditor Specifies information about the editor.
         */
        static create(pRenderer?: GridRenderer<Map>, pEditor?: IEditor): Map;
        /**
         * Creates a new map instance.
         * @param pRenderer Reference to a renderer for the map.
         * @param pEditor Specifies information about the editor.
         */
        constructor(pRenderer?: GridRenderer<Map>, pEditor?: IEditor);
        /** Retrieves information about the editor application. */
        readonly application: IEditor;
        /** Retrieves information about the editor used to create the definition. */
        readonly editor: IEditor;
        /** Retrieves the clusters. */
        readonly clusters: Clusters;
        /** Sets the map definition. */
        definition: IDefinition;
        /** Retrieves if the map is deserializing. */
        readonly isDeserializing: boolean;
        /**
         * Generates a serialized representation of a map.
         * @return Returns a reference to the serialized map.
         */
        serialize(): IDefinition & IGridResource;
        /**
         * Deserializes to a map.
         * @param pMap Specifies the serialized map.
         * @return Returns a reference to the instance.
         */
        deserialize(pMap: IDefinition & IGridResource): this;
        /**
         * Iterate recursively through each item in the map.
         * @param fnEach Callback which is invoked for each item.
         * @param sType Specifies the type of items.
         * @return Returns `true` if the iteration was interrupted.
         */
        forEach(
            fnItem:
                | ((pItem: Branch, nDepth: number) => boolean | void)
                | ((pItem: Cluster, nDepth: number) => boolean | void)
                | ((pItem: Node, nDepth: number) => boolean | void)
                | ((pItem: Condition, nDepth: number) => boolean | void)
                | ((
                      pItem: Branch | Cluster | Node | Condition,
                      nDepth: number
                  ) => boolean | void),
            sType?: "branches" | "clusters" | "nodes" | "conditions" | "all",
            pExclude?: Branch | Cluster | Node | Condition
        ): boolean;
        /** Complete reload of the map. */
        reload(): void;
        /** Destroys the map. */
        destroy(): void;
    }
}

declare module "@tripetto/map/lib/nodes/node" {
    import { GridNode } from "@marblecore/grid";
    import { INode } from "@tripetto/map/lib/nodes/interface";
    import { TNodeEvents } from "@tripetto/map/lib/nodes/events";
    import {
        INodeBlockType,
        TNodeBlock
    } from "@tripetto/map/lib/nodes/blocks/type";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { Slot } from "@tripetto/slots";
    import { Map } from "@tripetto/map/lib/map";
    import { MarkdownParser } from "@marblecore/markdown";
    import { IItemChangeEvent } from "@marblecore/matrix";
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    import { Nodes } from "@tripetto/map/lib/nodes/nodes";
    import { Stack } from "@marblecore/functional";
    import { IVariable } from "@tripetto/map/lib/variables";
    export type TNodeType =
        | "undefined"
        | "static"
        | "note"
        | "error"
        | TNodeBlock;
    export class Node extends GridNode<TNodeEvents> {
        /** Contains the explanation for the node. */
        explanation?: string;
        /** Retrieves the identifier of the node. */
        readonly id: string;
        /** Retrieves the parent node dispatcher. */
        readonly parent: Nodes | undefined;
        /** Retrieves the predecessor of the node. */
        readonly predecessor: Node | undefined;
        /** Retrieves the successor of the node. */
        readonly successor: Node | undefined;
        /** Retrieves the parent map. */
        readonly map: Map | undefined;
        /** Retrieves the parent cluster. */
        readonly cluster: Cluster | undefined;
        /** Retrieves the block for the node. */
        readonly block: NodeBlock | undefined;
        /** Retrieves the block type identifier as a string. */
        readonly blockTypeIdentifier: string;
        /** Retrieves if the node has a block. */
        readonly hasBlock: boolean;
        /** Retrieves if the block is invalid. */
        readonly isBlockInvalid: boolean;
        /** Specifies if the node is disabled. */
        disabled: boolean | undefined;
        /** Retrieves the node type. */
        readonly type: TNodeType;
        /** Sets the name of the node. */
        name: string | undefined;
        /** Specifies if the name of the node should be visible. */
        nameVisible: boolean | undefined;
        /** Sets the description of the node. */
        placeholder: string | undefined;
        /** Returns the raw label including markdown formatting. */
        readonly labelRaw: string;
        /** Retrieves a markdown instance for the label. */
        readonly labelMarkdown: MarkdownParser<IVariable>;
        /**
         * Retrieves the node label in plain text (all markdown is removed). This
         * label is the name of the node or the placeholder if no name is specified.
         */
        readonly label: string;
        /** Sets the description of the node. */
        description: string | undefined;
        /**
         * Fired when a node is changed.
         * @param pEvent Contains the event payload.
         * @event
         */
        protected onChange(pEvent: IItemChangeEvent<this>): void;
        /**
         * Calculates the dimensions of the node (only necessary when we are not moving).
         * @param nWidth Specifies the width of the node.
         * @param nHeight Specifies the height of the node.
         * @return Returns `true` if the node is resized.
         */
        resize(nWidth?: number, nHeight?: number): boolean;
        /**
         * Calculates the z-index of the resource.
         * @return Returns `true` if the z-index is changed.
         */
        zIndex(): boolean;
        /**
         * Assigns a block to the node.
         * @param pBlock Specifies the block type.
         * @return Returns a reference to the block instance.
         */
        assignBlock<T extends NodeBlock>(
            pBlock: INodeBlockType<T> | T
        ): T | undefined;
        /**
         * Unassigns a block.
         * @return Returns the previously assigned block or `undefined` if there
         * was none.
         */
        unassignBlock(): NodeBlock | undefined;
        /**
         * Generates a serialized representation of a node.
         * @return Returns a reference to the serialized data.
         */
        serialize(): INode;
        /**
         * Deserializes to a node.
         * @param pNode Contains the serialized node.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @return Returns a reference to the instance.
         */
        deserialize(pNode: INode, pAfterwards?: Stack): this;
        /**
         * Retrieves a slot.
         * @param sReference Specifies the slot reference or identifier.
         * @param pBlockTypeIdentifier Specifies the block type identifier of the node.
         * @return Returns a reference to the slot.
         */
        slot<S extends Slot>(
            sReference: string,
            pBlockTypeIdentifier?: string | TNodeBlock
        ): S | undefined;
        /** Refreshes the node (this will rerender the node and emit an `OnRename` event). */
        refresh(): this;
        /** Indicate the node is refreshed (this will emit an `OnRename` event). */
        refreshed(): this;
        /**
         * Inserts a new node before this node.
         * @return Returns a reference to the new node.
         */
        insertBefore(): Node;
        /**
         * Inserts a new node after this node.
         * @return Returns a reference to the new node.
         */
        insertAfter(): Node;
        /** Enables the node. */
        enable(): this;
        /** Disables the node. */
        disable(): this;
        /**
         * Duplicates the node.
         * @param pDest Specifies the destination node. If omitted a new node is inserted after this node.
         * @return Returns a reference to the new node.
         */
        duplicate(pDest?: Node): Node;
        /** Deletes this node from the map. */
        delete(): this;
        /** Destroys the node. */
        destroy(): void;
    }
}

declare module "@tripetto/map/lib/nodes/blocks/block" {
    /** Dependencies */
    import { Stack } from "@marblecore/functional";
    import { ISerialize } from "@marblecore/serializer";
    import { INodeBlock } from "@tripetto/map/lib/nodes/blocks/interface";
    import { INodeBlockType } from "@tripetto/map/lib/nodes/blocks/type";
    import { Node } from "@tripetto/map/lib/nodes/node";
    import { Slots } from "@tripetto/slots";
    import { Map } from "@tripetto/map/lib/map";
    export abstract class NodeBlock {
        /** Contains the type identifier of the block. */
        static readonly identifier: string;
        /** Contains the version of the block. */
        static readonly version: string;
        /** Contains the kind of block. */
        static readonly kind: "headless" | "ui";
        /** Contains the label localizer. */
        static readonly _: () => string;
        /** Retrieves the localized label (name) of the block. */
        static readonly label: string;
        /** Retrieves the block type. */
        readonly type: INodeBlockType<NodeBlock>;
        /**
         * Creates a new block instance or verifies an existing instance.
         * @param pBlock Reference to the block constructor or instance.
         * @param pNode Reference to the parent node.
         * @param bInitialize Specifies if the block should be initialized.
         * @return Returns the block instance.
         */
        static create<Block extends NodeBlock>(
            pBlock: INodeBlockType<Block> | Block | string,
            pNode: Node,
            bInitialize: boolean
        ): Block | undefined;
        /**
         * Constructs a new block.
         * @param pNode Reference to the node.
         */
        constructor(pNode: Node);
        /** Retrieves the identifier of the node. */
        readonly id: string;
        /** Retrieves the parent map. */
        readonly map: Map | undefined;
        /** Retrieves a reference to the node. */
        readonly node: Node;
        /** Retrieves a reference to the slots. */
        readonly slots: Slots;
        /** Retrieves the label of the node block. */
        readonly label: string;
        /** Retrieves if the block is initialized. */
        readonly isInitialized: boolean;
        /**
         * The block is assigned to a node.
         * @param pPrevious Reference to the previous block that was attached to the node.
         * @return Returns a reference to the block instance.
         */
        assignedToNode(pPrevious: NodeBlock | undefined): this;
        /** The block is unassigned from the node. */
        unassignedFromNode(): void;
        /** Marks a change in the block. */
        detectChange(): void;
        /**
         * Rerenders the block.
         * @param bShouldRerender Specifies if the node should rerender.
         * @event
         */
        rerender(bShouldRerender?: boolean): void;
        /**
         * Serializes a block.
         * @param pProperties Specifies additional serialized properties.
         * @return Returns a reference to the serialized block.
         */
        serialize(pProperties?: ISerialize): INodeBlock;
        /**
         * Deserializes to a block instance.
         * @param pBlock Contains the serialized block.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @return Returns a reference to the instance.
         */
        deserialize(pBlock: INodeBlock, pAfterwards: Stack): this;
        /** Destroys the block. */
        destroy(): void;
    }
}

declare module "@tripetto/map/lib/nodes/nodes" {
    /** Dependencies */
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { Stack } from "@marblecore/functional";
    import { GridNodes } from "@marblecore/grid";
    import { INode } from "@tripetto/map/lib/nodes/interface";
    import { INodeConstructor } from "@tripetto/map/lib/nodes/constructor";
    import { Node } from "@tripetto/map/lib/nodes/node";
    export class Nodes extends GridNodes<Node> {
        constructor(pNodeConstructor: INodeConstructor, pCluster: Cluster);
        /** Retrieves the cluster of the node dispatcher. */
        readonly cluster: Cluster;
        /**
         * Invoked when the nodes dimensions are changed.
         * @event
         */
        onDimensions(): void;
        /**
         * Inserts a new node.
         * @return Returns the new node.
         */
        append(): Node;
        /**
         * Inserts a new node.
         * @return Returns the new node.
         */
        insert(): Node;
        /**
         * Inserts a new node before the supplied node.
         * @param pNode Specifies the reference node.
         * @return Returns the new node.
         */
        insertBefore(pNode: Node): Node;
        /**
         * Inserts a new node after the supplied node.
         * @param pNode Specifies the reference node.
         * @return Returns the new node.
         */
        insertAfter(pNode: Node): Node;
        /**
         * Deserializes the supplied serialized nodes to the node dispatcher.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @param pNodes Specifies the nodes.
         * @return Returns a reference to the instance.
         */
        deserialize(pAfterwards: Stack, ...pNodes: INode[]): this;
    }
}

declare module "@tripetto/map/lib/branches/culling" {
    export type TBranchCulling = "first" | "each";
}

declare module "@tripetto/map/lib/branches/terminator" {
    export type TBranchTerminator = "continuous" | "abort" | "jump";
}

declare module "@tripetto/map/lib/variables" {
    /** Dependencies */
    import { Map } from "@tripetto/map/lib/map";
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    import { Slot } from "@tripetto/slots";
    export interface IVariable {
        /** Contains the variable key. */
        key: string;
        /** Label for the variable. */
        label?: string;
        /** Reference to a block. */
        block?: NodeBlock;
        /** Reference to a slot. */
        slot?: Slot;
        /** Reference to a slot group. */
        group?: Slot;
    }
    /**
     * Parses markdown variables.
     * @param pMap Specifies the source map.
     * @return Returns a parser object.
     */
    export const parseVariables: (
        pMap: Map | undefined
    ) => {
        numberOfChars: number;
        parser: (
            sKey: string
        ) =>
            | {
                  shift: number;
                  markdown: string;
                  value: IVariable | undefined;
              }
            | undefined;
    };
}

declare module "@tripetto/map/lib/observer" {
    import { Map } from "@tripetto/map/lib/map";
    export function add(map: Map): void;
    export function remove(pMap: Map): void;
    export function observe(map?: Map): void;
}

declare module "@tripetto/map/lib/decorators/affects/required" {
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    /**
     * Marks a property that affects the required condition of a node.
     * @param pTarget Reference to the target.
     * @param sProperty Contains the property.
     * @param pDescriptor Contains the property descriptor.
     */
    export function affectsRequired<T extends NodeBlock>(
        pTarget: T,
        sProperty: keyof T,
        pDescriptor?: PropertyDescriptor
    ): void;
    /**
     * Verifies if a required property evaluates to `true`.
     * @param pTarget Reference to the target.
     * @return Returns `true` if the target has required condition.
     */
    export function isRequired<T extends NodeBlock>(pTarget: T): boolean;
}

declare module "@tripetto/map/lib/namespaces" {
    import { NodeBlocksNamespace } from "@tripetto/map/lib/nodes/blocks/blocks";
    import { ConditionBlocksNamespace } from "@tripetto/map/lib/conditions/blocks/blocks";
    export interface IBlocksNamespace {
        readonly identifier: string;
        readonly nodeBlocks: NodeBlocksNamespace;
        readonly conditionBlocks: ConditionBlocksNamespace;
    }
    export class BlocksNamespaces {
        /** Retrieves the active namespace. */
        static readonly active: IBlocksNamespace;
        /** Retrieves a list of missing block type identifiers. */
        static readonly missingBlockTypes: string[];
        /** Retrieves the total number of erroneous blocks due to missing block types. */
        static readonly erroneousBlocks: number;
        /** Retrieves the namespace with the specified identifier. */
        static activate(sIdentifier: string): IBlocksNamespace;
        static activateDefault(): IBlocksNamespace;
        /** Retrieves the namespace for the specified identifier. */
        static get(sIdentifier: string): IBlocksNamespace;
        /** Retrieves if the supplied namespace is available. */
        static isAvailable(sIdentifier: string): boolean;
        /**
         * Load a namespace for the specified identifier.
         * @param sIdentifier Specifies the namespace identifier.
         * @param pContext Reference to a context to use.
         * @param sUMD UMD code (enclosed in a function) that is executed in global
         * context. It is important to supply a function as a string, where the first
         * and only argument of that function is a ref to the supplied context.
         * For example: `function(Tripetto){console.dir(Tripetto);}`
         * @return Returns a ref to the namespace.
         */
        static load(
            sIdentifier: string,
            pContext: {},
            sUMD: string
        ): IBlocksNamespace;
        /**
         * Loads a namespace for the specified identifier asynchronously.
         * @param sIdentifier Specifies the namespace identifier.
         * @param pContext Reference to a context to use.
         * @param fnLoad Load function with a done callback as argument that receives the UMD code (enclosed in a function) when the loading is done.
         * @param fnReady Invoked when the loading is done.
         */
        static loadAsync(
            sIdentifier: string,
            pContext: {},
            fnLoad: (fnDone: (sUMD: string) => void) => void,
            fnReady?: (pNamespace: IBlocksNamespace) => void
        ): void;
        /** Unloads the specified namespace. */
        static unload(sIdentifier: string): boolean;
        /**
         * Activate a namespace for the specified identifier (or tries to load it if it is not available).
         * @param sIdentifier Specifies the namespace identifier.
         * @param pContext Reference to a context to use.
         * @param sUMD UMD code (enclosed in a function) that is executed in global
         * context. It is important to supply a function as a string, where the first
         * and only argument of that function is a ref to the supplied context.
         * For example: `function(Tripetto){console.dir(Tripetto);}`
         * @return Returns a ref to the namespace.
         */
        static activateOrLoad(
            sIdentifier: string,
            pContext: {},
            sUMD: string
        ): IBlocksNamespace;
        /**
         * Activate a namespace for the specified identifier asynchronously (or tries to load it if it is not available).
         * @param sIdentifier Specifies the namespace identifier.
         * @param pContext Reference to a context to use.
         * @param fnLoad Load function with a done callback as argument that receives the UMD code (enclosed in a function) when the loading is done.
         * @param fnReady Invoked when the loading is done.
         */
        static activateOrLoadAsync(
            sIdentifier: string,
            pContext: {},
            fnLoad: (fnDone: (sUMD: string) => void) => void,
            fnReady?: (pNamespace: IBlocksNamespace) => void
        ): void;
        /** Resets all missing types. */
        static resetMissing(): void;
    }
}

declare module "@tripetto/map/lib/decorators/affects" {
    import { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    /** Marks a property that affects the name or label of a node or condition. */
    export function affects<T extends NodeBlock | ConditionBlock>(
        sTarget: "#name" | "#label"
    ): (
        pTarget: T,
        sProperty: string,
        pDescriptor?: PropertyDescriptor
    ) => void;
    /** Marks a property that affects the state of the condition. */
    export function affects<T extends ConditionBlock>(
        sTarget: "#condition"
    ): (
        pTarget: T,
        sProperty: string,
        pDescriptor?: PropertyDescriptor
    ) => void;
    /** Marks a property that affects the slots or the required condition of a node. */
    export function affects<T extends NodeBlock>(
        sTarget: "#slots" | "#required"
    ): (
        pTarget: T,
        sProperty: string,
        pDescriptor?: PropertyDescriptor
    ) => void;
}

declare module "@tripetto/map/lib/decorators/assigned" {
    /** Dependencies */
    import { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    /**
     * Marks the block assigned function that is invoked when the block is assigned
     * to a node or condition. The following arguments are passed to the function:
     * - `previous` (NodeBlock | ConditionBlock | undefined): Reference to any
     * previous block that was attached (could be undefined when there was none).
     */
    export function assigned<T extends NodeBlock | ConditionBlock>(
        pTarget: T,
        sMethod: keyof T,
        pDescriptor?: PropertyDescriptor
    ): void;
}

declare module "@tripetto/map/lib/decorators/block" {
    /** Dependencies */
    import { IConditionBlockDecorator } from "@tripetto/map/lib/conditions/blocks/decorator";
    import { INodeBlockDecorator } from "@tripetto/map/lib/nodes/blocks/decorator";
    import { TConditionBlock } from "@tripetto/map/lib/conditions/blocks/type";
    import { TNodeBlock } from "@tripetto/map/lib/nodes/blocks/type";
    /**
     * Defines a node block.
     * @param pProperties Specifies the properties for the block.
     */
    export function block(
        pProperties: INodeBlockDecorator
    ): (pBlock: TNodeBlock) => void;
    /**
     * Defines a condition block.
     * @param pProperties Specifies the properties for the block.
     */
    export function block(
        pProperties: IConditionBlockDecorator
    ): (pBlock: TConditionBlock) => void;
}

declare module "@tripetto/map/lib/decorators/created" {
    /** Dependencies */
    import { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    /**
     * Marks the block created function that is invoked when the block is created
     * (right after the instance is created and just before deserialization occurs).
     */
    export function created<T extends NodeBlock | ConditionBlock>(
        pTarget: T,
        sMethod: keyof T,
        pDescriptor?: PropertyDescriptor
    ): void;
}

declare module "@tripetto/map/lib/decorators/definition" {
    /**
     * Include property in the definition.
     * @param sMode Specifies the mode:
     * - `r`: Reads only a property from the definition;
     * - `w`: Writes only a property to the definition;
     * - `rw`: Reads and writes a property from/to the definition.
     */
    export function definition<T>(
        sMode: "r" | "w" | "rw"
    ): (
        pTarget: T,
        sProperty: string,
        pPropertyDescriptor?: PropertyDescriptor
    ) => void;
    /** Include property in the definition. */
    export function definition<T>(
        pTarget: T,
        sProperty: string,
        pPropertyDescriptor?: PropertyDescriptor
    ): void;
}

declare module "@tripetto/map/lib/decorators/destroyed" {
    /** Dependencies */
    import { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    /**
     * Marks the block destroyed function that is invoked when the block is
     * destroyed.
     */
    export function destroyed<T extends NodeBlock | ConditionBlock>(
        pTarget: T,
        sMethod: keyof T,
        pDescriptor?: PropertyDescriptor
    ): void;
}

declare module "@tripetto/map/lib/decorators/detached" {
    /** Dependencies */
    import { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    /**
     * Marks a function that is invoked when the supplied property is detached.
     * If your function returns a callback, the removal of the condition
     * block is canceled and the callback is invoked after all pending a-sync
     * operations on the map are done. Example: You have a condition block that is
     * attached to a node using the `Node` property of the condition block. When
     * the attached node is deleted, it is detached from the condition block and the
     * function is invoked.
     * @param sProperty Specifies the property to monitor.
     */
    export function detached<T extends ConditionBlock>(
        sProperty: keyof T
    ): (
        pTarget: T,
        sFunction: string,
        pDescriptor?: PropertyDescriptor
    ) => void;
}

declare module "@tripetto/map/lib/decorators/slots" {
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    /**
     * Marks the slot function where you define the slots for a block. The following
     * arguments are passed to the function:
     * - `slots` (SlotsOrchestrator): Reference to the `Slots` orchestrator.
     */
    export function slots<T extends NodeBlock>(
        pTarget: T,
        sMethod: keyof T,
        pDescriptor?: PropertyDescriptor
    ): void;
}

declare module "@tripetto/map/lib/decorators/unassigned" {
    /** Dependencies */
    import { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    /**
     * Marks the block unassigned function that is invoked when the block is
     * unassigned from a node.
     */
    export function unassigned<T extends NodeBlock | ConditionBlock>(
        pTarget: T,
        sMethod: keyof T,
        pDescriptor?: PropertyDescriptor
    ): void;
}

declare module "@tripetto/map/lib/decorators/upgraded" {
    /** Dependencies */
    import { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    /**
     * Marks the upgraded function that is invoked when a block is loaded that was
     * created using an older version of the block implementation. You can use an
     * upgrade function to migrate older blocks to the new version of the block.
     * The following arguments are passed to the function:
     * - `prevVersion` (string): String that contains the previous version number of
     * the block.
     */
    export function upgraded<T extends NodeBlock | ConditionBlock>(
        pTarget: T,
        sMethod: keyof T,
        pDescriptor?: PropertyDescriptor
    ): void;
}

declare module "@tripetto/map/lib/events" {
    /** Dependencies */
    import { Map } from "@tripetto/map/lib/map";
    import { IHookPayload } from "@marblecore/hookup";
    export interface IMapChangeEvent extends IHookPayload<"OnChange"> {
        readonly map: Map;
    }
    export interface IMapErrorEvent extends IHookPayload<"OnError"> {
        readonly map: Map;
        readonly erroneousBlocks: number;
        readonly missingBlockTypes: string[];
    }
}

declare module "@tripetto/map/lib/branches/events" {
    /** Dependencies */
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { IHookPayload } from "@marblecore/hookup";
    import { TBranchCulling } from "@tripetto/map/lib/branches/culling";
    import { TBranchTerminator } from "@tripetto/map/lib/branches/terminator";
    export type TBranchEvents =
        | "OnCullingChange"
        | "OnTerminatorChange"
        | "OnRename";
    export interface IBranchCullingEvent
        extends IHookPayload<"OnCullingChange"> {
        readonly branch: Branch;
        readonly culling: TBranchCulling;
    }
    export interface IBranchTerminatorEvent
        extends IHookPayload<"OnTerminatorChange"> {
        readonly branch: Branch;
        readonly terminator: TBranchTerminator;
        readonly jump: Cluster | undefined;
    }
}

declare module "@tripetto/map/lib/conditions/events" {
    /** Dependencies */
    import { Condition } from "@tripetto/map/lib/conditions/condition";
    import { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    import { IHookPayload } from "@marblecore/hookup";
    export type TConditionEvents =
        | "OnBlockAssign"
        | "OnBlockUnassign"
        | "OnBlockChange"
        | "OnRename"
        | "OnDisable";
    export interface IConditionBlockAssignEvent
        extends IHookPayload<"OnBlockAssign"> {
        readonly condition: Condition;
        readonly block: ConditionBlock;
    }
    export interface IConditionBlockUnassignEvent
        extends IHookPayload<"OnBlockUnassign"> {
        readonly condition: Condition;
        readonly block: ConditionBlock;
    }
    export interface IConditionBlockChangeEvent
        extends IHookPayload<"OnBlockChange"> {
        readonly condition: Condition;
        readonly block: ConditionBlock | undefined;
    }
}

declare module "@tripetto/map/lib/nodes/events" {
    /** Dependencies */
    import { IHookPayload } from "@marblecore/hookup";
    import { Node } from "@tripetto/map/lib/nodes/node";
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    export type TNodeEvents =
        | "OnBlockAssign"
        | "OnBlockUnassign"
        | "OnBlockChange"
        | "OnTypeChange"
        | "OnRename"
        | "OnDisable";
    export interface INodeBlockAssignEvent
        extends IHookPayload<"OnBlockAssign"> {
        readonly node: Node;
        readonly block: NodeBlock;
    }
    export interface INodeBlockUnassignEvent
        extends IHookPayload<"OnBlockUnassign"> {
        readonly node: Node;
        readonly block: NodeBlock;
    }
    export interface INodeBlockChangeEvent
        extends IHookPayload<"OnBlockChange"> {
        readonly node: Node;
        readonly block: NodeBlock | undefined;
    }
}

declare module "@tripetto/map/lib/branches/constructor" {
    /** Dependencies */
    import { Branch } from "@tripetto/map/lib/branches/branch";
    import { Branches } from "@tripetto/map/lib/branches/branches";
    export interface IBranchConstructor {
        new (pBranches: Branches): Branch;
    }
}

declare module "@tripetto/map/lib/clusters/events" {
    export type TClusterEvents = "OnRename";
}

declare module "@tripetto/map/lib/clusters/constructor" {
    /** Dependencies */
    import { Cluster } from "@tripetto/map/lib/clusters/cluster";
    import { Clusters } from "@tripetto/map/lib/clusters/clusters";
    export interface IClusterConstructor {
        new (pClusters: Clusters): Cluster;
    }
}

declare module "@tripetto/map/lib/conditions/blocks/labels" {
    /** Dependencies */
    import { ConditionBlock } from "@tripetto/map/lib/conditions/blocks/block";
    export class Labels {
        constructor(pBlock: ConditionBlock);
        /** Retrieves the branch label (if the branch is present). */
        readonly branch: string;
        /** Retrieves the branch label (if the branch is present and an orphan). */
        readonly branchOrphan: string;
        /** Retrieves the cluster label (if the cluster is present). */
        readonly cluster: string;
        /** Retrieves the cluster label (if the cluster is present and an orphan). */
        readonly clusterOrphan: string;
        /** Retrieves the node label (if the node is present). */
        readonly node: string;
        /** Retrieves the node label (if the node is present and an orphan). */
        readonly nodeOrphan: string;
    }
}

declare module "@tripetto/map/lib/conditions/blocks/context" {
    /** Dependencies */
    import { TNodeBlock } from "@tripetto/map/lib/nodes/blocks/type";
    export type TConditionBlockContext =
        | "*"
        | "cluster"
        | "branch"
        | "node"
        | TNodeBlock;
}

declare module "@tripetto/map/lib/conditions/constructor" {
    /** Dependencies */
    import { Condition } from "@tripetto/map/lib/conditions/condition";
    import { Conditions } from "@tripetto/map/lib/conditions/conditions";
    export interface IConditionConstructor {
        new (pConditions: Conditions): Condition;
    }
}

declare module "@tripetto/map/lib/nodes/constructor" {
    /** Dependencies */
    import { Node } from "@tripetto/map/lib/nodes/node";
    import { Nodes } from "@tripetto/map/lib/nodes/nodes";
    export interface INodeConstructor {
        new (pNodes: Nodes): Node;
    }
}

declare module "@tripetto/map/lib/nodes/blocks/blocks" {
    import { TNodeBlock } from "@tripetto/map/lib/nodes/blocks/type";
    export class NodeBlocksNamespace {
        /** Creates a new namespace instance. */
        constructor(sIdentifier: string);
        /** Retrieves the identifier for the namespace. */
        readonly identifier: string;
        /** Retrieves a list of registered blocks. */
        readonly all: TNodeBlock[];
        /** Retrieves the array of missing block types. */
        readonly missing: string[];
        /** Contains the number of missed block types */
        readonly errors: number;
        /**
         * Registers the supplied block.
         * @param pBlock Specifies the block.
         * @param sIdentifier Specifies the block type identifier.
         * @param pAliases Specifies type aliases for the block.
         * @param fnLabel Specifies the localized label for the block as a function.
         * @param sVersion Specifies the version of the block in SemVer format (defaults to `0.0.0`).
         * @param sKind Specifies the block kind. Can be one of the following values:
         * - `headless`: Specifies the block is headless (has no user interface);
         * - `ui`: Specifies the block implements a user interface.
         */
        register(
            pBlock: TNodeBlock,
            sIdentifier: string,
            pAliases: string[],
            fnLabel: () => string,
            sVersion: string,
            sKind: "headless" | "ui"
        ): TNodeBlock;
        /**
         * Retrieves the block constructor of the specified type identifier.
         * @param sIdentifier Specifies the block type identifier.
         * @return Returns the block constructor or `undefined` if the block
         * type is not available.
         */
        ofType<T extends TNodeBlock>(sIdentifier: string): T | undefined;
        /** Reset missing types. */
        resetMissing(): void;
    }
}

declare module "@tripetto/map/lib/conditions/blocks/blocks" {
    import { NodeBlock } from "@tripetto/map/lib/nodes/blocks/block";
    import { TConditionBlock } from "@tripetto/map/lib/conditions/blocks/type";
    import { TConditionBlockContext } from "@tripetto/map/lib/conditions/blocks/context";
    export class ConditionBlocksNamespace {
        /** Creates a new namespace. */
        constructor(sIdentifier: string);
        /** Retrieves the identifier for the namespace. */
        readonly identifier: string;
        /** Retrieves a list of registered blocks. */
        readonly all: TConditionBlock[];
        /** Retrieves the array of missing block types. */
        readonly missing: string[];
        /** Contains the number of missed block types */
        readonly errors: number;
        /**
         * Registers a block.
         * @param pBlock Specifies the block.
         * @param sIdentifier Specifies the block type identifier.
         * @param pAliases Specifies type aliases for the block.
         * @param fnLabel Specifies the localized label for the block as a function.
         * @param pContext Specifies the context.
         * @param sVersion Specifies the version of the block in SemVer format (defaults to `0.0.0`).
         */
        register(
            pBlock: TConditionBlock,
            sIdentifier: string,
            pAliases: string[],
            fnLabel: () => string,
            pContext: TConditionBlockContext | string,
            sVersion: string
        ): TConditionBlock;
        /**
         * Retrieves the registered blocks for the specified context.
         * @param pContext Specifies the context.
         */
        for<T extends TConditionBlock>(
            pContext: TConditionBlockContext | NodeBlock
        ): T[];
        /**
         * Retrieves the block constructor with the specified type identifier.
         * @param sIdentifier Specifies the block type identifier.
         * @return Returns the block constructor or `undefined` if the block
         * type is not available.
         */
        ofType<T extends TConditionBlock>(sIdentifier: string): T | undefined;
        /** Reset missing types. */
        resetMissing(): void;
    }
}
